'''
Convenient methods for file access

@package eWRT.access.file
Created on Dec 6, 2012

@author: albert
'''

from bz2 import BZ2File
from gzip import GzipFile
from os.path import basename


class CompressedFile(object):
    ''' An intelligent file object that transparently opens compressed
        files.
    '''
    COMPRESSION_EXT = ('bz2', 'gz')

    def __init__(self, fname, mode='rb'):  # ported to python3 (SV)
        self.name = fname
        if fname.endswith(".gz"):
            self.fhandle = GzipFile(fname, mode)
        elif fname.endswith(".bz2"):
            self.fhandle = BZ2File(fname, mode)
        else:
            self.fhandle = open(fname, mode)

    def __enter__(self):
        return self.fhandle

    def __exit__(self, e_type, e_value, e_traceback):
        self.fhandle.close()

    @classmethod
    def open(cls, fname, mode='rb'):  # ported to python3 (SV)
        return CompressedFile(fname, mode).fhandle

    @classmethod
    def get_extension_list(cls, fname):
        '''
        :param fname: the file name to analyze
        :rtype: a list of file extensions of this file
                ignoring extensions indicating file compression.

        e.g. 'x/y/test.awp.csv.bz2' -> ['csv', 'awp']
        '''
        ext_list = basename(fname).split(".")
        ext_list.reverse()
        return ext_list if ext_list[0] not in cls.COMPRESSION_EXT \
            else ext_list[1:]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@package: eWRT.ws.wordnet.tests

Convenience functions to access the NLTK WordNet interface
@author: Albert Weichselbraun
'''
from nose.tools import assert_equal

from eWRT.ws.wordnet import get_synonmys, get_antonyms, get_terms

# ------------------------------------------------------------------------------------
# Unit tests
# ------------------------------------------------------------------------------------

def test_get_antonyms():
    lemmas = get_antonyms('good') 
    assert_equal( get_terms( lemmas ), set( ('bad', 'badness', 'ill', 'evil', 'evilness') ) )



""" @package eWRT.util.execute
    Helpers for executing third party modules
"""

# (C)opyrights 2008-2012 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__revision__ = "$Id$"
__copyright__ = "GPL"

from subprocess import PIPE, Popen
from os.path import isfile


def pipe_content(cmd, stdin=None):
    """ Pipes the content through the given command.
        @param[in] cmd:   command to be executed
        @param[in] stdin: standard input
        @return: (exit_status, stdout)
    """
    if not isfile(cmd.split(" ")[0]):
        raise ValueError("Command {} is not available.".format(cmd))

    if stdin:
        process_stdin = PIPE
    else:
        process_stdin = None

    process_stdout = PIPE
    p = Popen(cmd.split(" "),
              bufsize=0,
              shell=False,
              stdin=process_stdin,
              stdout=process_stdout)

    # write input to stdin, if present
    if stdin:
        p.stdin.write(stdin)
        p.stdin.close()

    # get stdout
    content = p.stdout.read()
    return (p.wait(), content)


class TestExecute(object):

    def test_pipe_content(self):
        assert pipe_content("/bin/grep 22", "12\n22\n33") == (0, "22\n")



#!/usr/bin/env python

""" 
    @package DetectLanguage.reuters
    a generic method to retrieve text from the reuters corpus
"""

# -----------------------------------------------------------------------
# - (C)opyright 2009 by Albert Weichselbraun <albert@weichselbraun.net>
# -                    webLyzard technology gmbh <awe@weblyzard.com>
# -----------------------------------------------------------------------

__revision__ = "$Revision: 545 $"
__author__ = "Albert Weichselbraun"

import os

from xml.sax import handler, parseString
from glob import glob
from zipfile import ZipFile

from eWRT.config import REUTERS_DATA_DIR


class ReutersParser(handler.ContentHandler):

    RELEVANT_ELEMENTS = ('title', 'text')

    def __init__(self):
        self.clear()

    def startElement(self, name, attrs):
        if name.lower() in self.RELEVANT_ELEMENTS:
            self.isRelevantText = True

    def endElement(self, name):
        if name.lower() in self.RELEVANT_ELEMENTS:
            self.isRelevantText = False

    def characters(self, content):
        if self.isRelevantText:
            if content.strip():
                self.text.append(content.strip())

    def getText(self):
        return "\n".join(self.text)

    def clear(self):
        self.isRelevantText = False
        self.text = []


class Reuters(object):

    @staticmethod
    def getText(s):
        """ returns the relevant strings from the given reuters article
            @param[in] s      string containing the article
            @returns the text in the file from the headline and text fields
        """
        # parser = make_parser()
        rps = ReutersParser()
        parseString(s, rps)
        return rps.getText()


class ReutersGetCorpus(object):
    """ An iterator for all documents in the given language """

    def __init__(self, lang):
        self.files = glob(os.path.join(REUTERS_DATA_DIR, lang, "*.xml"))

    def __iter__(self):
        return self

    def next(self):
        if self.files:
            xmlTxt = open(self.files.pop()).read()
            return Reuters.getText(xmlTxt)
        else:
            raise StopIteration


class ReutersGetZipCorpus(object):
    """ An iterator over all documents in a zipped reuters corpus """

    def __init__(self, baseDir, lang):
        self.zipFiles = glob(os.path.join(baseDir, lang, "*.zip"))
        self.zip, self.files = self.openNextZipFile()

    def __iter__(self):
        return self

    def next(self):
        if not self.files:
            self.zip, self.files = self.openNextZipFile()

        fname = self.files.pop()
        return fname, Reuters.getText(self.zip.read(fname))

    def openNextZipFile(self):
        if not self.zipFiles:
            raise StopIteration

        fname = self.zipFiles.pop()
        zip = ZipFile(fname)

        return zip, zip.namelist()


if __name__ == '__main__':
    import sys
    for nr, t in enumerate(ReutersGetZipCorpus("./eval", "it")):
        print(t)
        sys.exit(0)

    for nr, t in enumerate(ReutersGetCorpus("de")):
        print nr



#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet.util
Provides utils and shortcuts for using ConceptNet

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""
from logging import getLogger, FileHandler, Formatter, INFO
from time import strftime
from operator import itemgetter

from eWRT.util.cache import MemoryCached
from eWRT.stat.language import STOPWORD_DICT
from eWRT.stat.string import VectorSpaceModel
from eWRT.ws.conceptnet import CONCEPTNET_BASE_URL
from eWRT.ws.conceptnet.lookup_result import LookupResult


LOGGER = getLogger("eWRT.ws.conceptnet.util")
LOGGER.setLevel(INFO)

hdlr = FileHandler("eWRT.ws.conceptnet.util.%s.log" %
                   (strftime("%Y-%m-%d_%H%M")))
hdlr.setFormatter(Formatter('%(asctime)s %(levelname)s %(message)s'))
LOGGER.addHandler(hdlr)

VALID_LANGUAGES = ('en', )
# only allow hypernym, hyponym and synonym relations for senses
VALID_SENSE_FILTER = [(u'rel', u'/r/IsA'),
                      (u'rel', u'/r/Synonym'), (u'rel', u'/r/InstanceOf')]
# requires at least 4 levels to describe the sense
# (e.g. /c/en/senes/x while /c/en/dog would fail this criteria)
MIN_SENSE_SPECIFICITY = 3

#MAX_CONTEXT_COUNT_SENSES = 100


def ground_term(term, input_context, pos_tag=None, stopword_list=STOPWORD_DICT['en'], limit=1):
    '''
    Grounds the given term to a ConceptNet url

    ::param term: the input term ground
    ::param input_context: a list of context terms used for the grounding
    ::param pos_tag: an optional pos_tag
    ::param limit: maximum number of concepts to return

    ::return: the ConceptNet url for the given input term
    '''
    context_vector = VectorSpaceModel(input_context)
    lookup_result = LookupResult(term=term, pos_tag=pos_tag)
    lookup_result.apply_language_filter(VALID_LANGUAGES)
    lookup_result.apply_edge_filter(VALID_SENSE_FILTER)

    senses = lookup_result.get_senses()
    LOGGER.info("Disambiguating senses for term '%s'", term.encode("utf8"))

    result = []
    for no, sense in enumerate(senses):
        if sense.specificity() < MIN_SENSE_SPECIFICITY:
            LOGGER.warn('Scipping sense #%d for %s: %s - to low specificity',
                        no, term, sense.url.encode("utf8"))
            continue

        sense_context = get_textual_data(sense, VALID_LANGUAGES, stopword_list)
        # if '/c/en/look_at' in sense.url:
        #    LOGGER.debug(context_result.edges)
        #    LOGGER.debug(sense, "&", ", ".join(sense_context.keys()))

        try:
            current_sim_score = context_vector * \
                VectorSpaceModel(sense_context)
        except ZeroDivisionError:
            # ignore empty contexts (e.g. due to words removed by the
            # stopword_list)
            continue

        result.append((sense, current_sim_score))

    return sorted(result, key=itemgetter(1), reverse=True)[:limit]


@MemoryCached(10000)
def get_textual_data(sense, valid_languages, stopword_list):
    '''
    ::returns: a vector space representation of the given concept
    '''
    context_result = LookupResult(
        conceptnet_url=CONCEPTNET_BASE_URL + "/" + sense.url.encode("utf8"), strict=True)
    context_result.apply_language_filter(valid_languages)
    LOGGER.info("Found %d assertions for sense '%s'.",
                len(context_result.edges), sense)
    return context_result.get_vsm(stopword_list)


if __name__ == '__main__':
    ground_term('lord', ['hallo'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 25.09.2012

@author: Norman Süsstrunk, Heinz-Peter Lang, Albert Weichselbraun
'''
from __future__ import print_function

import os
import unittest
import logging
import dateutil.parser as dateparser
import urllib
import json
import pytz

from six import string_types
from operator import attrgetter
from datetime import datetime, timedelta
from cPickle import dump
from time import sleep
from apiclient.discovery import build

from eWRT.ws.WebDataSource import WebDataSource


logger = logging.getLogger('eWRT.ws.youtube')

DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.000Z'


def convert_date(str_date): return datetime.strptime(str_date, DATE_FORMAT)


def get_value(key, dictionary):
    if '.' in key and not key.isdigit():
        old_key, new_key = key.split('.', 1)
        if old_key in dictionary:
            return get_value(new_key, dictionary[old_key])
    else:
        if key in dictionary:
            return dictionary[key]

    return None


class YouTubeEntry(dict):

    DATE_FIELDS = ('published', 'last_modified')
    YOUTUBE_SEARCH_URL = 'https://www.youtube.com/watch?v='

    VIDEO_MAPPING = {
        'id': 'video_id',
        'comments': None,
        'channel': None,
        'snippet.title': 'title',
        'snippet.publishedAt': 'published',
        'snippet.description': 'description',
        'snippet.channelId': 'user_id',
        'snippet.thumbnails.default.url': 'thumbnail',
        'contentDetails.duration': 'duration',
        'contentDetails.caption': 'caption',
        'contentDetails.licensedContent': 'licensed',
        'statistics.viewCount': 'statistics_viewcount',
        'statistics.favoriteCount': 'statistics_favoritecount',
        'statistics.likeCount': 'statistics_likecount',
        'statistics.dislikeCount': 'statistics_dislikecount',
        'statistics.commentCount': 'statistics_commentcount',
        'topicDetails.relevantTopicIds': 'freebase_topics_relevant',
        'topicDetails.topicIds': 'freebase_topics',
    }

    COMMENT_MAPPING = {
        'id': 'comment_id',
        'parent': 'parent_id',
        'snippet.authorDisplayName': 'user_name',
        'snippet.authorGoogleplusProfileUrl': 'user_profile',
        'snippet.viewer_rating': 'viewer_rating',
        'snippet.likeCount': 'like_count',
        'snippet.totalReplyCount': 'reply_count',
        'snippet.channelId': 'channel_id',
        'snippet.publishedAt': 'published',
        'snippet.updatedAt': 'last_modified',
        'snippet.textDisplay': 'content',
    }

    CHANNEL_MAPPING = {
        'id': 'user_id',
        'kind': 'user_type',
        'snippet.description': 'user_status',
        'snippet.title': 'user_name',
        'snippet.country': 'user_location',
        'published_at': 'location'
    }

    def __init__(self, search_result, mapping=VIDEO_MAPPING):
        ''' constructor initializes entry with the result'''
        dict.__init__(self)
        self.update(self.update_entry(search_result, mapping=mapping))

        if 'video_id' in self:
            self['url'] = ''.join([self.YOUTUBE_SEARCH_URL, self['video_id']])

    def __repr__(self):
        return '** entry: %s' % '\n'.join(['%s: %s' % (k, v) for k, v in self.iteritems()])

    def update_entry(self, search_result, mapping=VIDEO_MAPPING):
        ''' stores the mapped items in a dictionary
        @param search_result: the search result returned by youtube
        @param entry_type: string, either comment or video
        '''
        for attr, key in mapping.iteritems():

            if attr == 'comments' and 'comments' in search_result:
                self['comments'] = search_result['comments']
                continue

            if attr == 'channel' and 'channel' in search_result:
                self['channel'] = search_result['channel']
                continue

            value = get_value(attr, search_result)
            if isinstance(value, unicode):
                value = value.encode('utf-8')
            if key in self and isinstance(self[key], list):
                if value:
                    self[key].append(value)
            else:
                self[key] = value

        # parse dates
        if 'published' in self and not 'last_modified' in self:
            self['last_modified'] = self['published']

        for date_field in self.DATE_FIELDS:
            if date_field in self and isinstance(self[date_field], string_types):
                self[date_field] = dateparser.parse(self[date_field],
                                                    ignoretz=True)
        return self


class YouTube_v3(WebDataSource):

    YOUTUBE_API_SERVICE_NAME = 'youtube'
    YOUTUBE_API_VERSION = 'v3'

    '''
    number of seconds to wait between comment request
    this has been added to prevent the youtube API from
    blocking us via quota per seconds
    '''
    YOUTUBE_SLEEP_TIME = 1
    MAX_RESULTS_PER_QUERY = 50

    FREEBASE_SEARCH_URL = 'https://www.googleapis.com/freebase/v1/search?%s'

    def __init__(self, api_key, max_comment_count=10, get_details=True,
                 sleep_time=YOUTUBE_SLEEP_TIME):
        self.api_key = api_key
        self.max_comment_count = max_comment_count
        self.get_details = get_details
        self.client = build(self.YOUTUBE_API_SERVICE_NAME,
                            self.YOUTUBE_API_VERSION,
                            developerKey=self.api_key)
        self.sleep_time = sleep_time

    @classmethod
    def _get_yt_dict(cls, entry, mapping):
        """ stores the mapped items in a dictionary
        @param entry: Gdata Entry
        @param mapping: dictionary with the mapping
        @return: yt_dict
        """
        yt_dict = {}
        for attr, key in mapping.iteritems():
            try:
                yt_dict[key] = attrgetter(attr)(entry)

                if key in ('published', 'last_modified'):
                    yt_dict[key] = convert_date(yt_dict[key])

            except AttributeError as e:
                logger.warn('AttributeError: %s' % e)
                yt_dict[key] = None

        return yt_dict

    def get_channelId_for_user(self, user_name):
        ''' 

        https://www.googleapis.com/youtube/v3/channels?key={YOUR_API_KEY}&forUsername={USER_NAME}&part=id
        '''
        result = self.client.channels().list(part='id,snippet',
                                             forUsername=user_name)
        return result

    def get_video_search_feed(self, search_terms, max_results=25, max_comment_count=0):
        """
        Returns a generator of youtube search results
        """
        for search_result in self.search(search_terms, max_results):
            if search_result['id']['kind'] == 'youtube#video':
                try:
                    yield self._convert_item_to_video(search_result,
                                                      max_comment_count)
                except Exception as e:
                    logger.error(
                        'Failed to convert Youtube search result: %s' % e)

    def _get_video_rating(self, video_id):
        """ Returns the rating for a video ID """
        return self.client.videos().getRating(id=video_id).execute()

    def _get_video_status(self, video_id):
        return self.client.videos().list(id=video_id,
                                         part='id').execute()

    def _get_video_details(self, video_id):
        return self.client.videos().list(id=video_id,
                                         part='contentDetails,statistics,topicDetails').execute()

    def _get_video_channel_detail(self, channel_id):
        return self.client.channels().list(id=channel_id,
                                           part='contentOwnerDetails,snippet').execute()

    def like_video(self, video_id):
        """ Adds to the video rating. This code sets the rating to "like," but you
            could also support an additional option that supports values of "like"
            and "dislike"."""
        self.client.videos().rate(id=video_id, rating="like").execute()

    def _build_youtube_item(self, item, max_comment_count=0, get_details=False):
        """ """

        video_id = item['id']['videoId']
        channel_id = item['snippet']['channelId']

        # retrieve comments and details as requested
        if max_comment_count > 0:
            item['comments'] = [YouTubeEntry(comment, YouTubeEntry.COMMENT_MAPPING)
                                for comment in self._get_video_comments(video_id=video_id)]

        if get_details:
            details = self._get_video_details(video_id=video_id)['items']
            if details and len(details) > 0:
                item.update(details[0])

            channel_info = self._get_video_channel_detail(
                channel_id=channel_id)['items']
            if channel_info and len(channel_info) > 0:
                item['channel'] = YouTubeEntry(
                    channel_info[0], YouTubeEntry.CHANNEL_MAPPING)

        return YouTubeEntry(item)

    def username_to_channels(self, username):
        """
        Retreive the channel_id of a youtube user
        @param username, username of youtube account
        """
        response = self.client.channels().list(
            forUsername=username, part='id').execute()
        for item in response['items']:
            yield item['id']

    def search(self, search_terms, since_date=None, region_code=None, language=None,
               max_results=MAX_RESULTS_PER_QUERY, is_channel=False):
        """ 
        Search the Youtube API for videos matching a set of search terms 

        @param search_terms, a comma separated list of search terms
        @param max_results, the maximum number of results returned
        @param max_age, the maximum number of days ago from which to include results
        @param region_code, ISO 3166-1 alpha-2 country code, e.g. AT, DE, GB, US
        @param language, ISO 639-1 two-letter language code, e.g. de, cs, en
        """
        if not since_date:
            since_date = datetime.utcnow() - timedelta(days=1)
            since_date = since_date.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        if isinstance(since_date, datetime):
            local = pytz.timezone("Europe/Vienna")
            local_dt = local.localize(since_date, is_dst=None)
            utc_dt = local_dt.astimezone(pytz.utc)
            since_date = utc_dt.strftime('%Y-%m-%dT%H:%M:%S.%fZ')

        items_per_page = min([max_results, self.MAX_RESULTS_PER_QUERY])
        kwargs = {
            'part': 'id,snippet',
            'type': 'video',
            'publishedAfter': since_date,
            'maxResults': items_per_page,
            'order': 'date'}
        if is_channel:
            kwargs['channelId'] = search_terms
        else:
            kwargs['q'] = search_terms

        if region_code:
            kwargs['regionCode'] = region_code
        if language:
            kwargs['relevanceLanguage'] = language

        continue_search = True
        items_count = 0

        while continue_search:

            response = self.client.search().list(**kwargs).execute()
            total_results = response['pageInfo']['totalResults']

            for search_result in response.get('items', []):
                if search_result['id']['kind'] == 'youtube#video':
                    try:
                        items_count += 1
                        yield self._build_youtube_item(search_result,
                                                       max_comment_count=self.max_comment_count,
                                                       get_details=self.get_details)
                    except Exception as e:
                        logger.error('Failed to convert Youtube item: %s' % e)

            if items_count >= max_results:
                continue_search = False

            if items_count >= total_results:
                continue_search = False

            if items_count == 0:
                continue_search = False

            if not 'nextPageToken' in response:
                continue_search = False
            else:
                kwargs['pageToken'] = response['nextPageToken']

    def _get_video_comments(self, video_id):
        """ Returns the comments for a youtube ID"""
        result = []
        try:
            comments = self.client.commentThreads().list(part='snippet',
                                                         videoId=video_id,
                                                         textFormat='plainText'
                                                         ).execute()
        except Exception:
            return result  # just ignore, comments might be disabled

        if not comments or not 'items' in comments:
            return result  # just ignore, comments might be disabled

        for item in comments['items']:
            result.append(item['snippet']['topLevelComment'])

        return result

    def get_freebase_topics(self, QUERY_TERM):
        """ Retrieves a list of Freebase topics associated with the query term """
        freebase_params = dict(query=QUERY_TERM, key=self.api_key)
        freebase_url = self.FREEBASE_SEARCH_URL % urllib.urlencode(
            freebase_params)
        freebase_response = json.loads(urllib.urlopen(freebase_url).read())
        if len(freebase_response['result']) == 0:
            print('No matching terms were found in Freebase.')

        return freebase_response['result']



#!/usr/bin/env python

# (C)opyrights 2008-2009 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz-Peter Lang <heinz@langatium.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

import re
import unittest
import optparse
import sys
import logging

try:
    from io import StringIO
except:
    import StringIO

import time
from datetime import datetime, timedelta, date

try:
    from urllib.parse import urlparse, parse_qs  # porting to python 3.4 (SV)
except:
    from urlparse import urlparse, parse_qs  # python2

from xml.dom.minidom import parse, parseString
from lxml import etree

from eWRT.access.http import Retrieve

SEARCH_URL = 'http://www.google.com/search?hl=en&ie=UTF-8&q={searchTerm}&num={number}&safe=active&start={start}'
#
XPATHS = {'blog_url': './h3[@class="r"]/a',
          'blog_date': './div[@class="s"]/span[@class="f"]',
          'search_result': './/div[@id="ires"]/ol',
          'blog_abstract': './div[@class="s"]/text()'}

# Google displays a maximum 100 results per page
MAX_RESULTS_PAGE = 100
SLEEP_TIME = 10
SUPPORTED_COUNTRIES = ('AT', 'DE')

logger = logging.getLogger('logger')


class GoogleBlogSearch(object):
    ''' implements functions for accessing Google's Blogsearch '''

    @staticmethod
    def get_content(url, sleep_time=SLEEP_TIME):
        ''' fetches the content
        @param url: url to fetch
        @param sleep_time: time to sleep
        @return: HTML string'''
        assert(url.startswith("http"))
        return Retrieve("GoogleBlogSearch",
                        sleep_time=sleep_time).open(url).read()

    @staticmethod
    def get_blog_links(searchTerm, maxResults=100, offset=0, maxAge=0,
                       country=None):
        ''' returns a list of URLs
        @param searchTerm:
        @param maxResult:
        @param offset:
        @param maxAge:
        @param country: country code, e.g. AT, DE, ...
        @return:
        '''

        if isinstance(searchTerm, list):
            searchTerm = '+'.join(searchTerm)

        searchTerm = re.sub(' ', '+', searchTerm)

        if maxAge > 0:
            dateFormat = '%m/%d/%Y'
            max = date.today().strftime(dateFormat)
            min = (date.today() - timedelta(days=maxAge)).strftime(dateFormat)
            maxAgeString = '&tbs=cdr:1,cd_min:{min},cd_max:{max}'.format(
                min=min, max=max)
        else:
            maxAgeString = ''

        url = SEARCH_URL.format(searchTerm=searchTerm, start=offset,
                                number=maxResults, maxAge=maxAgeString)

        if country:
            if country.upper() in SUPPORTED_COUNTRIES:
                url = '%s&cr=country%s' % (url, country.upper())
                url = url.replace('.com/', '.%s/' % country.lower())
            else:
                logger.error('Do not recognize country "%s"' % country)

        logger.debug('Searching URL %s' % url)
        html_content = GoogleBlogSearch.get_content(url)
#         print html_content
        tree = etree.HTML(html_content)
        resultList = tree.xpath('.//div[@id="ires"]/ol')

        counter = 0
        firstElement = True
        urls = []

        for element in resultList[0].iterchildren():
            if firstElement:
                firstElement = False
            else:
                itemList = element.xpath('./h3[@class="r"]/a')
                if len(itemList) == 1:
                    url = itemList[0].attrib['href']
                    abstract = ' '.join(element.xpath(
                        './div[@class="s"]/text()'))
                    url = GoogleBlogSearch.parse_url(url)

                    if url:
                        blogLink = {}
                        blogLink['url'] = url
                        blogLink['source'] = 'GoogleBlogSearch - Keyword "%s"' % searchTerm
                        blogLink['abstract'] = abstract
                        blogLink['reach'] = '0'

                        try:
                            blogLink['date'] = GoogleBlogSearch.get_link_date(element)
                        except IndexError as e:
                            pass

                        urls.append(blogLink)

                        counter += 1

                    if (counter + offset) >= maxResults:
                        break

        if maxResults > (MAX_RESULTS_PAGE) and (counter + offset) < maxResults:

            urls.extend(GoogleBlogSearch.get_blog_links(searchTerm,
                                                        maxResults=maxResults, offset=(
                                                            counter + offset),
                                                        maxAge=maxAge))
        return urls

    @staticmethod
    def parse_url(url):

        if url.startswith('/'):
            url = 'www.google.com%s' % url

        o = urlparse(url)
        query = parse_qs(o.query)

        correct_url = None

        if not 'q' in query:
            logger.critical('URL %s does not contain the parameter "q"' % url)
        else:
            if isinstance(query['q'], list):
                correct_url = query['q'][0]
            elif isinstance(query['q'], list):
                correct_url = query['q'][0]
            else:
                logger.critical(
                    'Unknown type "%s" for query["q"]' % type(query['q']))

        return correct_url

    @staticmethod
    def get_link_date(element):
        linkDate = element.xpath('./div[@class="f"]/text()')[0]
        linkDate = linkDate.split('by')[0]

        m = re.match('(\d{1,2} \w* \d{2,4})', linkDate)

        if m:
            linkDate = datetime.strptime(m.groups()[0], '%d %b %Y')
        else:
            now = datetime.today()
            if 'ago' in linkDate:

                tdelta = None
                if 'day' in linkDate:
                    days = int(re.split(' ', linkDate)[0])
                    tdelta = timedelta(days=days)
                elif 'hour' in linkDate:
                    hours = int(re.split(' ', linkDate)[0])
                    tdelta = timedelta(hours=hours)
                elif 'minute' in linkDate:
                    minutes = int(re.split(' ', linkDate)[0])
                    tdelta = timedelta(minutes=minutes)

                linkDate = now - tdelta

        return linkDate



#!/usr/bin/env python
'''

'''
import logging
import pyvirtualdisplay

from time import sleep
from selenium import webdriver

from eWRT.access import abstract_retrieve


__version__ = '0.0.8'

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SetUpOpenAndTearDown(object):

    def __init__(self, set_up_func, tear_down_func):
        self.set_up_func = set_up_func
        self.tear_down_func = tear_down_func

    def __call__(self, function):
        def _wrapper(*args, **kwargs):

            self.set_up_func(*args, **kwargs)
            function(*args, **kwargs)
            self.tear_down_func(*args, **kwargs)
        return _wrapper


class JavascriptRetriever(abstract_retrieve.AbstractRetriever):
    '''
    Emulates a X screen and controls a Firefox instance.

    For using the JavascriptRetriever you need to install **xvfb**, **xephyr** and **Firefox**.
    The JavascriptRetriever object is a singleton. In order to fully close it,
    please use the method `tear_down()`.

    Usage
        >>> retriever = JavascriptRetriever()
        >>> website_str = retriever.open('http://www.weblyzard.com')
        >>> retriever.tear_down()
    '''
    num_selenium_retrievers = 0
    display = None
    browser = None

    def __init__(self, waiting_time_in_secs=30):
        '''
        :param float waiting_time_in_secs: Describes how long the Firefox should maximally wait until it raises an
                                           exception.
        '''
        self.waiting_time_in_secs = waiting_time_in_secs
        self._initialize_display()

    def _initialize_display(self):
        if not self.display:
            self.display = pyvirtualdisplay.Display()
            self.display.start()

        if not self.browser:
            self.browser = webdriver.Firefox()
            self.browser.implicitly_wait(self.waiting_time_in_secs)
            self.num_selenium_retrievers += 1

    def set_up(self, *args, **kwargs):
        self._initialize_display()

    def tear_down(self, *args, **kwargs):
        '''
        Closes Firefox, X and removes the instance.
        '''
        self.browser.quit()
        self.display.stop()
        del self

    def open(self, url, **kwargs):
        '''
        :param str url: Surf to a given URL and return the source code of the site.
        '''
        self.browser.get(url=url)
        sleep(2)
        return self.browser.page_source





class WebDataSource(object):
    """ A WebDataSource performs search queries against Web Sources such as Youtube, Twitter, ... """

    def search(self, search_terms):
        """ returns the count for the given tags
            @param search_terms: a list of search terms
            @returns a list of matching documents """
        return NotImplemented



""" helper utilities required to create databases
    for the modules in this package
    
    (*) create-distance-db.py
        - transforms wikipedia dumps into the database
          format required for the distance module.
"""


#!/usr/bin/env python

import abc


class AbstractRetriever(object):

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def set_up(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def tear_down(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def open(self):
        pass



#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet
Access to conceptnet data structures using its REST interface

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""


class Edge(dict):
    '''
    A ConceptNet edge.
    '''

    def __init__(self, edge_dict):
        '''
        ::param edge_dict: a dictionary used to initialize the edge
        '''
        dict.__init__(self)
        self.update(edge_dict)

# IMPORTANT: block antonyms and scores with negative values
# NLP is not yet ready to handle them



# #!/usr/bin/env python
# 
# # (C)opyrights 2008-2009 by Albert Weichselbraun <albert@weichselbraun.net>
# #                           Heinz-Peter Lang <heinz@langatium.net>
# # 
# # This program is free software: you can redistribute it and/or modify
# # it under the terms of the GNU General Public License as published by
# # the Free Software Foundation, either version 3 of the License, or
# # (at your option) any later version.
# # 
# # This program is distributed in the hope that it will be useful,
# # but WITHOUT ANY WARRANTY; without even the implied warranty of
# # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# # GNU General Public License for more details.
# # 
# # You should have received a copy of the GNU General Public License
# # along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# __version__ = "$Header$"
# 
# import time
# import re, logging, sys
# 
# from lxml import etree
# from datetime import timedelta, date
# 
# from eWRT.access.http import Retrieve
# from eWRT.ws.TagInfoService import TagInfoService
# 
# 
# SLEEP_TIME = 5
# NON_ABSTRACT_TEXT = ['a', 'h3']
# RESULTS_PER_PAGE = 10
# 
# cleanText = lambda text: (re.sub('\n', ' ', text)).strip()
# 
# logger = logging.getLogger('Technorati')
# handler = logging.StreamHandler()
# logger.addHandler(handler)
# 
# class Technorati(TagInfoService):
#     """ retrieves data using the del.icio.us API """
# 
#     RE_TAG_COUNT = re.compile('Posts relating to &ldquo;.*?&rdquo; \((\S+)\)',
#         re.IGNORECASE | re.DOTALL)
#     RE_TAG_CONTAINER = re.compile('<div id="related-tags".*?<ul>(.*?)</ul>.*?</div>',
#         re.IGNORECASE | re.DOTALL)
#     RE_RELATED_TAG = re.compile('<a.*?">(.*?)</a>',
#         re.IGNORECASE | re.DOTALL)
# 
#     AVAILABLE_TOPICS = ['overall', 'entertainment', 'business', 'sports', 'politics', 'autos', 'technology', 'living', 'green', 'science']
#     AVAILABLE_AUTHORITY = ['all', 'high', 'medium', 'low']
#     AVAILABLE_SOURCE = ['advanced-source-blogs', 'advanced-source-news', 'advanced-source-all']
#     AVAILABLE_RETURN = ['posts', 'news']
# 
#     TOPIC = 'overall'
#     AUTHORITY = 'high'
#     SOURCE = 'advanced-source-all'
#     RETURN = 'posts'
# 
#     TECHNORATI_URL = 'http://technorati.com/search?usingAdvanced=1&q=%s&return=%s&source=%s&topic=%s&authority=%s'
# 
#     __slots__ = ()
#     last_access = 0
# 
# 
#     @staticmethod
#     def getTagInfo(tags):
#         """ @param   tags   A list of tags to retrieve information for
#             @returns        the number of bookmarks using the given tags
#         """
#         url = Technorati._parseURL(tags)
#         content = Technorati.get_content(url)
# 
#         return Technorati._parse_tag_counts(content)
# 
# 
#     @staticmethod
#     def getRelatedTags(tags, withCounts=True):
#         """ @param tags - list of tags
#             @param booelan withCounts to print counts for tags
#             @return list of related tags 
#         """
# 
#         # not supported at the moment
# 
#         return []
# 
#         url = Technorati._parseURL(tags)
# 
#         content = Technorati.get_content(url)
#         tag_container = Technorati.RE_TAG_CONTAINER.findall(content)
#         related_tags_with_count = []
# 
#         if len(tag_container) > 0:
# 
#             related_tags = Technorati.RE_RELATED_TAG.findall(tag_container[0])
#             if withCounts:
#                 for tag in related_tags:
#                     tag = re.sub(' ', '-', tag)
#                     related_tags_with_count.append((tag, Technorati._getRelatedTagCount(tags, tag)))
# 
#                 return related_tags_with_count
# 
#             else:
#                 return related_tags
# 
# 
# 
#     # 
#     # helper functions
#     #
#     @staticmethod
#     def _parse_tag_counts(content):
#         """ parses technorati html content and returns the number of counts for the tags 
#             @param content: HTML content of technorati
#             @return: count of related posts
#         """
# 
#         m = Technorati.RE_TAG_COUNT.search(content)
#         if m:
#             return re.sub(',', '', m.group(1))
#         else:
#             return 0
# 
#     @staticmethod
#     def _getRelatedTagCount(tags, filter):
#         """ returns the related tag count
#             @param tags - list of tags
#             @param filtered tag
#             @return count of related tag """
# 
#         url = Technorati._parseURL(tags, filter)
#         content = Technorati.get_content(url)
#         return Technorati._parse_tag_counts(content)
# 
#     @staticmethod
#     def _parseURL(tags, filter=''):
#         """ parses the URL """
# 
#         if type(tags).__name__ == 'str':
#             url = Technorati.TECHNORATI_URL % (tags, Technorati.RETURN, Technorati.SOURCE, Technorati.TOPIC, Technorati.AUTHORITY)
#         else:
#             url = Technorati.TECHNORATI_URL % ("+".join(tags), Technorati.RETURN, Technorati.SOURCE, Technorati.TOPIC, Technorati.AUTHORITY)
# 
#         if filter == '':
#             return url
#         else:
#             return '%s&filter=%s' % (url, filter)
# 
# 
#     @staticmethod
#     def get_content(url):
#         """ returns the content from Technorati """
#         assert(url.startswith("http"))
# 
#         logger.debug('Fetching content for URL %s' % url)
# 
#         if (time.time() - Technorati.last_access) < SLEEP_TIME:
#             logger.debug('Sleeping %s seconds!' % SLEEP_TIME)
#             time.sleep(SLEEP_TIME)
# 
#         Technorati.last_access = time.time()
# 
#         f = Retrieve("%s_new" % Technorati.__name__).open(url)
#         content = f.read()
#         f.close()
#         return content
# 
#     @staticmethod
#     def get_blog_links (searchTerm, maxResults=100, offset=0, maxAge=0, 
#                         country=None):
#         '''
#         Searches Technorati for the given term and returns a list of links 
#         to the blogs
#         @param searchTerm: term to search
#         @param maxResults: max search results 
#         @param offset:
#         @param maxAge: maximum age of the articles in days,
#             if set links are ordered by relevance and not by date
#         @param country: currently not supported
#         @return: dictionary with all the relevant info
#         '''
# 
#         if isinstance(searchTerm, list):
#             searchTerm = '+'.join(searchTerm)
# 
#         searchTerm = re.sub(' ', '+', searchTerm)
#         if offset >= RESULTS_PER_PAGE:
#             page = (offset / RESULTS_PER_PAGE) + 1
#         else:
#             page = 1
# 
#         if maxAge == 0: orderBy = 'relevance'
#         else: orderBy = 'date'
# 
#         url = '%s&page=%s&sort=%s' % (Technorati._parseURL(searchTerm), 
#                                       page, orderBy)
#         content = Technorati.get_content(url)
#         print '.'
#         return Technorati._parseLinksFromContent(content, searchTerm, 
#                                                  maxResults=maxResults, 
#                                                  offset=offset, maxAge=maxAge)
# 
# 
#     @staticmethod
#     def _parseLinksFromContent(content, searchTerm, maxResults=100, 
#                                offset=0, maxAge=0):
#         ''' parses the links from the content '''
#         tree = etree.HTML(content)
#         links = []
#         resultList = tree.xpath('//ol[@class="search-results post-list"]')
# 
#         logger.debug('Parsing links from Technorati result: %s, %s, %s, %s ' % (searchTerm, maxResults, offset, maxAge))
# 
#         if len(resultList) > 0:
#             counter = 0
#             for element in resultList[0].iterchildren():
#                 blogLink = {}
#                 linkDate = Technorati._getDate(element, maxAge)
#                 if not linkDate == None:
# 
#                     try:
#                         blogLink['url'] = element.xpath('.//a[@class="offsite"]')[0].attrib['href']
#                         logger.info('Found url %s' % blogLink['url'])
#                     except IndexError:
#                         logger.error('Could not parse the URL')
# 
#                     try:
#                         blogLink['authority'] = int(re.sub('Authority ', '', element.xpath('.//a[@class="authority"]')[0].text))
#                     except:
#                         logger.error('Could not parse the authority!')
#                         blogLink['authority'] = ''
# 
#                     blogLink['source'] = 'Technorati - Keyword "%s"' % searchTerm
#                     blogLink['abstract'] = Technorati._getAbstract(element)
#                     blogLink['reach'] = '0'
#                     blogLink['date'] = linkDate
# 
#                     if 'url' in blogLink:
#                         logger.debug('Found URL %s' % blogLink['url'])
#                         links.append(blogLink)
#                         offset += 1
#                         counter += 1
#                         if offset == maxResults:
#                             break
#                     else:
#                         offset += 1
# 
#             if offset < maxResults and counter == RESULTS_PER_PAGE:
#                 links.extend(Technorati.get_blog_links(searchTerm, maxResults, 
#                                                        offset, maxAge))
# 
#         else:
#             if offset == 0:
#                 logger.warning('At the first page - should find a result for %s -> Changes in the site structure' % searchTerm)
# 
#         return links
# 
#     @staticmethod
#     def _getAbstract(element):
#         ''' gets the abstract from the element '''
#         text = []
# 
#         if not element.tag in NON_ABSTRACT_TEXT and not element.text == None:
#             text.append(element.text.lstrip())
# 
#         for child in element.getchildren():
#             text.extend(Technorati._getAbstract(child))
# 
#         if not element.tag in NON_ABSTRACT_TEXT and not element.tail == None:
#             text.append(element.tail.lstrip())
# 
#         return ''.join(text)
# 
#     @staticmethod
#     def _getDate(element, maxAge=0):
#         ''' returns the date of the link '''
# 
#         p = re.compile('(\d+)\s(\w+)\sago')
# 
#         minDate = (date.fromtimestamp(time.time()) - timedelta(days=maxAge))
#         linkDate = None
#         for div in element.xpath('.//div'):
#             m = p.search(div.text)
#             if m:
#                 age = int(m.groups()[0])
#                 timeDim = m.groups()[1]
# 
#                 if re.search('week.?', timeDim):
#                     hours = age * 7 * 24
#                 elif re.search('day.?', timeDim):
#                     hours = age * 24
#                 elif re.search('hour.?', timeDim):
#                     hours = age
#                 elif re.search('minute.?', timeDim):
#                     hours = 1
#                 else:
#                     hours = 0
# 
#                 if hours > 0:
#                     linkDate = date.fromtimestamp(time.time()) - timedelta(hours=hours)
# 
#                     if linkDate < minDate and maxAge > 0:
#                         linkDate = None
#             else:
#                 logger.warning("Could not find a date")
# 
#         return linkDate
# 
# 
# if __name__ == '__main__':
# 
#     print Technorati.getRelatedTags(("Linux",))



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on May 29, 2018

@author: Max Göbel <goebel@weblyzard.com>
'''
import httplib2
import unittest
import datetime

from time import sleep
from datetime import date
from apiclient.discovery import build

from oauth2client.client import OAuth2Credentials


class GAnalyticsClient():

    VERSION = 'v3'  # needed by apiclient
    SERVICE_NAME = 'analytics'  # needed by apiclient

    QUERY_THROTTLE_SECS = 1  # sleep interval between queries in seconds
    MAX_RESULTS = '25'

    metrics = ['ga:visitors', 'ga:newVisits', 'ga:percentNewVisits',
               'ga:sessions', 'ga:bounces', 'ga:entranceBounceRate',
               'ga:bounceRate', 'ga:sessionDuration', 'ga:avgSessionDuration',
               ]

    def __init__(self, credentials, check_new=False):
        ''' '''
        self.service = None
        self.credentials = credentials
        self.source_group_name = 'ganalytics'
        self.webproperties = None

        if not self.service:
            self.service = self.init_service(self.credentials)

        # update DB
        if check_new:
            self.update_profile_db()

    @classmethod
    def init_service(cls, credentials):
        ''' '''
        http = httplib2.Http()

        cred = OAuth2Credentials(access_token=credentials['access_token'],
                                 client_id=credentials['client_id'],
                                 client_secret=credentials['client_secret'],
                                 refresh_token=credentials['refresh_token'],
                                 token_expiry=credentials['token_expiry'],
                                 token_uri=credentials['token_uri'],
                                 user_agent=credentials['user_agent'])

        http = cred.authorize(http)

        # Construct and return the authorized Analytics Service Object
        return build(cls.SERVICE_NAME, cls.VERSION, http=http)

    def get_webproperty_details(self, account_id, webproperty_id):
        ''' '''
        profiles = self.service.management().profiles().list(
            accountId=account_id,
            webPropertyId=webproperty_id).execute()

        for profile in profiles.get('items'):
            yield profile.get('id')

    def get_metrics(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns core metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        metrics = ','.join(self.metrics)
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            metrics=metrics).execute()

    def get_country_split(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns geo metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            metrics='ga:visits',
                                            dimensions='ga:country',
                                            sort='-ga:visits',
                                            filters='ga:medium==organic',
                                            start_index='1',
                                            max_results=self.MAX_RESULTS).execute()

    def get_top_keywords(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns keyword metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            metrics='ga:visits',
                                            dimensions='ga:keyword',
                                            sort='-ga:visits',
                                            filters='ga:medium==organic',
                                            start_index='1',
                                            max_results='25').execute()

    def get_page_track(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns page track metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            metrics='ga:visits',
                                            dimensions='ga:hostname,ga:pagePath',
                                            sort='-ga:visits',
                                            filters='ga:medium==organic',
                                            start_index='1',
                                            max_results='25').execute()

    def get_social_media(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns social media metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            metrics='ga:visits',
                                            dimensions='ga:socialNetwork',
                                            sort='-ga:visits',
                                            filters='ga:medium==organic',
                                            start_index='1',
                                            max_results='25').execute()

    def get_top_referers(self, profile_id, start_date=None, end_date=None):
        '''
            Executes and returns referer metrics from the 
            Google Core Reporting API.
            Args:
                optional start and end date
            Returns:
                The response returned from the Core Reporting API.
        '''
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")
        if isinstance(start_date, date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, date):
            end_date = end_date.strftime("%Y-%m-%d")
        return self.service.data().ga().get(ids='ga:' + profile_id,
                                            start_date=start_date,
                                            end_date=end_date,
                                            dimensions='ga:source',
                                            metrics='ga:pageviews,ga:sessionDuration,ga:exits',
                                            filters='ga:medium==referral',
                                            sort='-ga:pageviews',
                                            start_index='1',
                                            max_results='25').execute()

    def get_accounts(self):
        ''' '''
        return self.service.management().accounts().list().execute().get('items')

    def get_profiles(self, account_id):
        ''' Get active profiles configured for a Google Analytics account. '''
        result = []
        webproperties = self.service.management().webproperties().list(
            accountId=account_id).execute()

        for item in webproperties.get('items'):
            for pid in self.get_webproperty_details(self.service,
                                                    account_id,
                                                    item.get('id')):
                result.append((account_id, pid))
        return result

    def process_profile(self, profile_id, account_id, start_date=None, end_date=None):
        ''' 
        Process a single google analytics profile by id
        '''
        # set start and end date if None
        if not start_date:
            start_date = datetime.date.today().strftime("%Y-%m-%d")
        if not end_date:
            end_date = datetime.date.today().strftime("%Y-%m-%d")

        result = {}

        # query top country split
        countries = self.get_country_split(profile_id=profile_id,
                                           start_date=start_date,
                                           end_date=end_date)

        str_arr = []
        if 'rows' in countries:
            for row in countries['rows']:
                str_arr.append(row[0] + ':' + row[1])
        result['ga:countries'] = ','.join(str_arr)

        # query top search keywords
        sleep(self.QUERY_THROTTLE_SECS)
        keywords = self.get_top_keywords(profile_id=profile_id,
                                         start_date=start_date,
                                         end_date=end_date)
        str_arr = []
        if 'rows' in keywords:
            for row in keywords['rows']:
                str_arr.append(row[0] + ':' + row[1])
        result['ga:keywords'] = ','.join(str_arr)

        # query top referers
        sleep(self.QUERY_THROTTLE_SECS)
        referers = self.get_top_referers(profile_id=profile_id,
                                         start_date=start_date,
                                         end_date=end_date)
        str_arr = []
        if 'rows' in referers:
            for row in referers['rows']:
                str_arr.append(row[0] + ':' + row[1])
        result['ga:referers'] = ','.join(str_arr)

        sleep(self.QUERY_THROTTLE_SECS)
        soc_media = self.get_social_media(profile_id=profile_id,
                                          start_date=start_date,
                                          end_date=end_date)
        str_arr = []
        if 'rows' in soc_media:
            for row in soc_media['rows']:
                if row[0] and row[0] != '(not set)':
                    str_arr.append(row[0] + ':' + row[1])
        result['ga:social_media'] = ','.join(str_arr)

        sleep(self.QUERY_THROTTLE_SECS)
        page_track = self.get_page_track(profile_id=profile_id,
                                         start_date=start_date,
                                         end_date=end_date)
        page_arr = []
        if 'rows' in page_track:
            for row in page_track['rows']:
                if row[1] and row[1] != '(not set)':
                    page_arr.append(row[1] + ':' + row[2])
        result['ga:page_track'] = ','.join(page_arr)

        # get metrics defined in params
        sleep(self.QUERY_THROTTLE_SECS)
        response = {}
        webproperties = self.service.management().webproperties().list(
            accountId=account_id).execute()
        for item in webproperties.get('items'):
            sleep(self.QUERY_THROTTLE_SECS)
            for pid in self.get_webproperty_details(service=self.service,
                                                    account_id=account_id,
                                                    webproperty_id=item.get('id')):
                if pid == profile_id:
                    response = self.get_metrics(profile_id=profile_id,
                                                start_date=start_date,
                                                end_date=end_date)
                    break
        i = 0
        if response.get('columnHeaders'):
            for key in response.get('columnHeaders'):
                if 'rows' in response:
                    value = response.get('rows')[0][i]
                key_str = key.get('name')
                i = i + 1
                result[key_str] = value
        return result


class GanalyticsTest(unittest.TestCase):

    def test_ganalytics(self):

        # Get stored credentials or run the Auth Flow if none are found
        metrics = ['ga:visitors', 'ga:newVisits', 'ga:percentNewVisits',
                   'ga:sessions', 'ga:bounces', 'ga:entranceBounceRate',
                   'ga:bounceRate', 'ga:sessionDuration', 'ga:avgSessionDuration',
                   ]

        client = GAnalyticsClient()
        result = client.run(",".join(metrics))
        print result


if __name__ == "__main__":
    unittest.main()



# #!/usr/bin/env python
# # -*- coding: utf-8 -*-
# import simplejson
# import oauth2
# 
# from atom import Link
# from six import string_types
# 
# from eWRT.ws.WebDataSource import WebDataSource
# 
# 
# class LinkedIn(WebDataSource):
#     
#     def __init__(self, consumer_key, consumer_secret, user_token, user_secret):
#         WebDataSource.__init__(self)
#         
#         consumer = oauth2.Consumer(consumer_key, consumer_secret)
#         access_token = oauth2.Token(key=user_token, secret=user_secret)
#         self.client = oauth2.Client(consumer, access_token) 
#         self.linkedin_uri = """http://api.linkedin.com/v1/"""
#         
#     def search(self, search_terms, companies=False, jobs=False, option='keywords', 
#                     format='json'):
#         
#         '''Search linkedin for companies or jobs.
#         The number of search results is limited to 10 items per search term.
#         
#         Usage::
#         >>> linkedin_obj = LinkedIn(consumer_key, consumer_secret, user_token, user_secret)
#         >>> linkedin_obj.search(['IBM', 'Microsoft'], companies=True)
#         >>> linkedin_obj.search(['Python', 'Machine Learning'], jobs=True)
#         
#         If you need a more specific search you can use certain options instead
#         of the option keywords or you place a custom search request::
#         
#         >>> linkedin_obj.search(['Python', 'Machine Learning'], jobs=True, option='job-title')
#         >>> linkedin_obj.custom_search('http://api.linkedin.com/v1/company-search:(facets)?keywords={Python}&facets=location')
#         '''
#         assert isinstance(search_terms, list)
#         assert not (companies and jobs) and (companies or jobs)
#         
#         retval = None
#         
#         if companies:
#             retval = self.search_companies(search_terms, format)
#         elif jobs:
#             retval = self.search_job_by_search_terms(search_terms, format)
#         
#         assert retval
#         return retval
# 
#     def custom_search(self, uri, format='json'):
#         self._assert_string(uri)
#         self._assert_format(format)
#         
#         response, content = self.client.request(uri, 
#                                                 headers={'x-li-format':format})
#         
#         return simplejson.loads(content)
# 
#     def search_company(self, search_term, option='keywords', format='json'):
#         '''Search for companies using one search term'''
#         self._assert_string(search_term, option)
#         self._assert_format(format)
#         
#         request = self.linkedin_uri + 'company-search?%s=%s' % (option,
#                                                                 search_term)
#         response, content = self.client.request(request, 
#                                                 headers={'x-li-format':format})
#             
#         return self._convert_json_to_dict(content, 'companies')
#     
#     
#     def search_companies(self, search_terms, option='keywords', format='json'):
#         '''Search for companies using several keywords'''
#         
#         assert isinstance(search_terms, list)
#         self._assert_string(option)
#         self._assert_format(format)
#         
#         results = []
#         for search_term in search_terms:
#             results.append(self.search_company(search_term, option, format))
#             
#         return results
#     
#     
#     def search_job_by_search_term(self, search_term, option='keyword', 
#                                   format='json'):
#         '''Search for a job using a certain search term'''
#         
#         self._assert_string(search_term, option)
#         self._assert_format(format)
#         
#         request = self.linkedin_uri + 'job-search?%s=%s' % (option, search_term)
#         response, content = self.client.request(request, 
#                                                 headers={'x-li-format':format})
#         
#         return self._convert_json_to_dict(content, search_field='jobs')
#     
#     def search_job_by_search_terms(self, search_terms, option='keywords', 
#                                    format='json'):
#         '''Search for jobs using several search terms.'''
#         self._assert_format(format)
#         assert isinstance(search_terms, list)
#         self._assert_string(option)
#         
#         results = []
#         for search_term in search_terms:
#             results.append(self.search_job_by_search_term(search_term, option, 
#                                                           format))
#         
#         return results
# 
#     def _convert_json_to_dict(self, json_string, search_field):
#         self._assert_string(json_string, search_field)
#         
#         raw_data = simplejson.loads(json_string)
# 
#         results = []        
#         for key, value in raw_data.items():
#             
#             if key == search_field:
#                 for item in value['values']:
#                     results.append(item)
#         
#         return results
# 
#     def _assert_format(self, format):
#         assert format == 'json' or format == 'xml'
#         
#     def _assert_string(self, *args):
#         for possible_string in args:
#             assert isinstance(possible_string, string_types)
#             assert len(possible_string) > 0



'''
Created on 21.06.2012

@author: Norman Suesstrunk, Heinz-Peter Lang

class for executing batch requests to the facebook api
'''
import json
import urllib
import httplib
import logging

from ssl import SSLError

from eWRT.config import FACEBOOK_ACCESS_KEY
from eWRT.ws.facebook import FacebookWS
from eWRT.ws.exceptions import AuthenticationError


MAX_BATCH_SIZE = 3
TIMEOUT = 100  # timeout for the requests in seconds

logger = logging.getLogger('eWRT.ws.facebook')


class FbBatchRequest(object):
    '''
    @class FacebookBatchRequest
    Class for sending batch requests to the facebook api
    the actual requests are constructed with objects from the FacebookWS class.  
    '''

    batchHTTPParam = 'batch'  # http post parameter for the facebook batch http interface
    accessTokenHTTPParam = 'access_token'
    faceBookGraphHost = 'graph.facebook.com'

    def __init__(self, access_token=FACEBOOK_ACCESS_KEY):
        ''' Constructor '''
        self.access_token = access_token

    def run_search(self, terms, objectType='all', since=None, limit=None):
        ''' runs a batch search '''
        if not isinstance(terms, list):
            terms = [terms]

        return self.make_search([FacebookWS(term, objectType,
                                            since, limit) for term in terms])

    def make_search(self, fbWSList):
        ''' executes the search 
        :param fbWSList: list of :class:eWRT.ws.facebook.FacebookWS object
        :type fbWSList: list
        :returns: result
        :rtype: list
        '''

        assert len(fbWSList), 'list of facebook services empty'

        result = []
        search_result = self._send_post(self.access_token, fbWSList)

        for row in search_result:
            if not row:
                logger.debug('row == %s ... continue' % row)
                continue
            elif row.get('body', None) is None:
                logger.debug("the row's body is non-existent or None")
                continue
            try:
                data = json.loads(row['body'])
            except KeyError as e:
                if 'error' in row and\
                   'Error validating access token' in row['error'].get('message', ''):
                    raise AuthenticationError(str(row['error']))
                else:
                    raise e

            if 'data' in data:
                for post in json.loads(row['body'])['data']:
                    post['url'] = 'http://www.facebook.com/%s' % post['id']
                    result.append(post)
                    if 'comments' in post and 'data' in post['comments']:
                        for i, comment in enumerate(post['comments']['data']):
                            comment['type'] = 'comment'
                            comment['parent_url'] = post['url']
                            comment['url'] = '%s?comment=%s' % (post['url'], i)
                            result.append(comment)
            elif data:
                result.append(data)

        return result

    @classmethod
    def _get_json_batch_request_string(cls, fbWSList):
        '''
        delivers the json-string in the apropriate format for the facebook batch api
        '''
        return [fb.getJsonListStructure() for fb in fbWSList]

    @classmethod
    def _send_post(cls, access_token, fbWSList):
        '''
        sends the batch request as post to the facebook batch request api
        returns all the search results for the batch request
        '''
        result = []
        conn = httplib.HTTPSConnection(cls.faceBookGraphHost,
                                       timeout=TIMEOUT)
        all_batch_requests = cls._get_json_batch_request_string(fbWSList)

        for batch_requests in cls.get_batch(all_batch_requests):
            batch_requests = json.dumps(batch_requests)
            logger.debug('Making batch_requests with %s' % batch_requests)
            params = urllib.urlencode({cls.accessTokenHTTPParam: access_token,
                                       cls.batchHTTPParam: batch_requests})
            headers = {'Content-type': 'application/x-www-form-urlencoded',
                       'Accept': 'text/plain'}

            try:

                conn.request('POST', '/', params, headers)
                response = conn.getresponse()

                data = json.loads(response.read())

                if isinstance(data, dict) and 'error' in data:
                    result.append(data)
                else:
                    result.extend(data)

            except SSLError, e:
                logger.error('Could not request: %s, %s' % (batch_requests, e))

        conn.close()
        return result

    @staticmethod
    def get_batch(requests, batch_size=MAX_BATCH_SIZE):
        for i in range(0, len(requests), batch_size):
            yield requests[i:i + batch_size]



#!/usr/bin/env python
"""
@package eWRT.access.db` -- database access
========================================
"""

# (C)opyrights 2004-2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


__author__ = "Albert Weichselbraun"
__revision__ = "$Revision$"


from types import StringTypes
from warnings import warn
try:
    import psycopg2
    import psycopg2.extras
except ImportError:
    warn("Cannot import postgresql library.")
try:
    import MySQLdb
except ImportError:
    warn("Cannot import mysql library.")

# logging
import logging
log = logging.getLogger(__name__)


class IDB(object):
    """ @interface IDB
        database access interface

        @remarks
        supports the context protocal
     """

    def connect(self):
        """ connects to the database """

    def query(self, qu):
        """ processes a queries to the database and returns
            the result.
            @param[in] query (list or string)
            @returns the result as dictionary
        """
        raise NotImplementedError

    def close(self):
        """ closes the database connection """
        raise NotImplementedError

    def __enter__(self):
        """ support fo the context protocol """
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """ context protocol support """
        if exc_type != None:
            log.critical("Database error: %s" % exc_type)
        self.close()


class MysqlDb(IDB):

    def __init__(self, dbname, host="", username="", passwd="", connect=True):
        """ @param[in] connect immediately connect to the database
        """
        self.dbname = dbname
        self.host = host
        self.username = username
        self.passwd = passwd

        if connect:
            self.connect()

    def connect(self):
        self.db = MySQLdb.connect(
            host=self.host, user=self.username, passwd=self.passwd, db=self.dbname)

    def query(self, query):
        """ queries the database and stores the result in a dict """
        crs = self.db.cursor(MySQLdb.cursors.DictCursor)

        if type(query) in StringTypes:
            query = (query,)
        for q in query:
            crs.execute(q)
        tmp = crs.fetchall()
        crs.close()
        return tmp

    def close(self):
        self.db.close()


class PostgresqlDb(IDB):
    """ @class  PostgresqlDb
        provides generall database access """

    __db = {}           # cache db connections
    DEBUG = False

    def __init__(self, dbname, host="", username="", passwd="", multiThreaded=True, connect=True):
        """ inits the database class 
            @param[in] multiThreaded specifies whether the connection will be used
                                     in a multi-threaded environment.
        """
        self.dbname = dbname
        self.host = host
        self.username = username
        self.passwd = passwd
        self.db = None
        self.multiThreaded = multiThreaded
        if connect:
            self.connect()

    def connect(self):
        """ connects to the database

            @remarks 
            caches the connection if multiThreaded is False; the connection caching does not
            work in multi-threaded environments
        """
        if self.multiThreaded:
            self.db = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'" % (
                self.dbname, self.username, self.host, self.passwd))

        else:
            dbKey = (self.dbname, self.username, self.host, self.passwd)
            if dbKey not in self.__db:
                self.__db[dbKey] = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'" % (
                    self.dbname, self.username, self.host, self.passwd))
            self.db = self.__db[dbKey]

    def query(self, qu):
        """ @param[in] qu a list or string containing the database quer(y|ies)
            @returns the query results
         """
        if type(qu) in StringTypes:
            qu = (qu,)
        if PostgresqlDb.DEBUG:
            log.debug("Query: %s" % qu)
        cur = self.db.cursor(cursor_factory=psycopg2.extras.DictCursor)
        [cur.execute(q) for q in qu]
        return cur.fetchall()

    def execute(self, q):
        cur = self.db.cursor(cursor_factory=psycopg2.extras.DictCursor)
        return cur.execute(q)

    def getCursor(self):
        return self.db.cursor()

    def commit(self):
        self.db.commit()

    def close(self):
        self.db.close()



#!/usr/bin/env python

''' @package eWRT.util.timing
    timing of abitrary method calls

    Examples: see unittests
'''

# (C)opyrights 2010 - 2015 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Albert Weichselbraun"
__copyright__ = "GPL"

from time import time
from unittest import TestCase


class Timed(object):
    ''' decorator class used to time functions '''

    def __init__(self, f):
        self.f = f
        self.clear()

    def __call__(self, *args, **kargs):
        ''' calls the timed function and computes all necessary
            statistics '''
        self.numberOfCalls += 1
        beforeCall = time()
        retVal = self.f(*args, **kargs)
        self.lastCallDuration = time() - beforeCall
        self.totalCallDuration += self.lastCallDuration
        return retVal

    def clear(self):
        self.startTime = time()
        self.lastCallDuration = None
        self.totalCallDuration = 0.
        self.numberOfCalls = 0


class TimedTest(TestCase):

    @Timed
    def _timeFunction(self, a, b):
        [x for x in range(1000)]
        return a + b

    def testReturnValue(self):
        assert self._timeFunction(self, 2, 8) == 10

    def testCallStatistics(self):
        NR_CALL = 5000
        for x in range(NR_CALL):
            self._timeFunction(self, x, 10 * x)

        assert self._timeFunction.numberOfCalls == NR_CALL
        self.assertAlmostEqual(self._timeFunction.totalCallDuration /
                               NR_CALL, self._timeFunction.lastCallDuration, 1)

    def testClearCallStatistics(self):
        NR_CALL = 5000
        for x in range(NR_CALL):
            self._timeFunction(self, x, 10 * x)

        self._timeFunction.clear()
        assert self._timeFunction.totalCallDuration == 0.
        assert self._timeFunction.numberOfCalls == 0



#!/usr/bin/env python
"""
 @package eWRT.ws.stat.string.spelling
 suggests spelling corrections for arbitrary terms
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                   and others (as outlined in the functions.
#
#
# The code published in this module is either under the GNU General
# Public License (see below) or under the license specified in the
# function.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import collections

from string import ascii_lowercase
from bz2 import BZ2File
from cPickle import dump, load
from os.path import dirname, join as pjoin

from logging import getLogger

log = getLogger(__name__)

DEFAULT_MODEL = pjoin(dirname(__file__), "eWRT.stat.string.spelling.data.bz2")
valid_word_characters = ascii_lowercase + "-"


class SpellSuggestion(object):
    """
    @class SpellSuggestion
    learns a vocabulary and returns spelling suggestions for
    deviating words.

    This code is based on code presented in a tutorial written by 
    Peter Norvig <http://norvig.com/spell-correct.html>.
    """

    model = collections.defaultdict(lambda: 1)

    def __init__(self, serializedModel=DEFAULT_MODEL, verbose=False):
        """
        @param[in] filename of the cPickle dump of the model
        """
        if serializedModel:
            self.model.update(load(BZ2File(serializedModel)))

        self.verbose = verbose

    def serializeModel(self, serializedModel):
        """
        Writes the learned model to a compressed file.
        @param[in] serializedModel file name for the serialized model
        """
        f = BZ2File(serializedModel, "w")
        dump(dict(self.model), f)
        f.close()

    @staticmethod
    def words(text): return re.findall('[a-z]+', text.lower())

    def train(self, features):
        """ @param[in] features a list of terms to consider as correct
                                for the spell checking (including repetitions as
                                they increase a words probability to get selected
                                as spelling suggestion.
        """
        for f in features:
            self.model[f] += 1

    @staticmethod
    def edits1(word):
        splits = [(word[:i], word[i:]) for i in range(len(word) + 1)]
        deletes = [a + b[1:] for a, b in splits if b]
        transposes = [a + b[1] + b[0] + b[2:] for a, b in splits if len(b) > 1]
        replaces = [a + c + b[1:]
                    for a, b in splits for c in valid_word_characters if b]
        inserts = [a + c + b for a, b in splits for c in valid_word_characters]
        return set(deletes + transposes + replaces + inserts)

    def known_edits2(self, word):
        return set(e2 for e1 in self.edits1(word) for e2 in self.edits1(e1) if e2 in self.model)

    def known(self, words): return set(w for w in words if w in self.model)

    def correct(self, word):
        """
        Determines whether the spelling is correct (i.e. known) and
        suggests the most likely candidate otherwise
        @param[in] word the word to check
        @returns (correct, suggestion) a tuple indicating whether the spelling is correct
                  and a suggestion for the correct spelling
        """

        # we found the word in the dictionary
        # or ignore words containing "/" and numbers
        if word in self.model:
            return (True, word)
        elif "/" in word or not word.isalpha():
            return (False, word)

        candidates = self.known(self.edits1(
            word)) or self.known_edits2(word) or [word]
        suggestion = max(candidates, key=self.model.get)
        if self.verbose and word != suggestion:
            log.debug("Replacing '%s' with '%s'" % (word, suggestion))
        return (False,  suggestion)


class TestSpellSuggestion(object):

    def __init__(self):
        self.s = SpellSuggestion()

    def testSpellSuggestion(self):
        assert self.s.correct("determinned") == (False, "determined")
        assert self.s.correct("teh") == (False, "the")
        assert self.s.correct("runnign") == (False, "running")

        # return the same term for correct word
        assert self.s.correct("environment") == (True, "environment")

        # return the original term if we cannot find a better suggestion
        assert self.s.correct("weichselbraun") == (False, "weichselbraun")
        assert self.s.correct("rangersdorf") == (False, "rangersdorf")

    def testExceptions(self):
        assert self.s.correct("co2") == (False, "co2")
        assert self.s.correct("i/o") == (False, "i/o")


if __name__ == '__main__':
    pass
    # s = SpellSuggestion(DEFAULT_MODEL)
    #s.train( SpellSuggestion.words( BZ2File('spell-training-corpus.text.bz2').read() ) )
    # s.serializeModel(DEFAULT_MODEL)



#!/usr/bin/env python
'''
 @package eWRT.ws.stat.string
 String statistics such as
  * word similarity
  * levenshtein distance
  * soundex
  * ...
'''

# (C)opyrights 2010 - 2015 by Albert Weichselbraun <albert@weichselbraun.net>
#                          and others (as outlined in the functions).
#
# The code published in this module is either under the GNU General
# Public License (see below) or under the license specified in the
# function.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import math
try:
    from itertools import izip_longest as zip_longest
except ImportError:
    from itertools import zip_longest
from operator import mul, itemgetter
from collections import Counter
from numpy import vdot
from numpy.linalg import norm

from eWRT.util.cache import MemoryCached
from eWRT.lib.thirdparty.advas.phonetics import caverphone, metaphone, nysiis


def wordSimilarity(s1, s2, similarityMeasure):
    ''' computes the given similarity metric on a strings
        words.
        e.g. wordSimilarity("nuclear energy", "energy nuclear", lev) = 0

        missing words are replaced by ""
        e.g. wordSimilarity("as you thought", "you thought") ==
               wordSimilarity("as you thought", "you thought", "")

    '''
    wordList = list(zip_longest(s1.split(), s2.split(), fillvalue=""))
    words1, words2 = map(itemgetter(0), wordList), map(itemgetter(1), wordList)
    assert len(words1) == len(words2)

    score = float(sum([min([similarityMeasure(w1, w2) for w1 in words1])
                       for w2 in words2])) / len(words1)
    return score


@MemoryCached
def lev(s1, s2):
    ''' Levenshtein string edit distance
        source:
        http://en.wikibooks.org/wiki/Algorithm_implementation/Strings/Levenshtein_distance#Python
    '''
    if len(s1) < len(s2):
        return lev(s2, s1)
    if not s1:
        return len(s2)

    previous_row = xrange(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            # j+1 instead of j since previous_row and current_row are one
            # character longer
            insertions = previous_row[j + 1] + 1
            deletions = current_row[j] + 1       # than s2
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row

    return previous_row[-1]


@MemoryCached
def damerauLev(seq1, seq2):
    '''
    Calculate the Damerau-Levenshtein distance between sequences.
    source : http://mwh.geek.nz/2009/04/26/python-damerau-levenshtein-distance/
    license: MIT license


    This distance is the number of additions, deletions, substitutions,
    and transpositions needed to transform the first sequence into the
    second. Although generally used with strings, any sequences of
    comparable objects will work.

    Transpositions are exchanges of *consecutive* characters; all other
    operations are self-explanatory.

    This implementation is O(N*M) time and O(M) space, for N and M the
    lengths of the two sequences.

    >>> dameraulevenshtein('ba', 'abc')
    2
    >>> dameraulevenshtein('fee', 'deed')
    2

    It works with arbitrary sequences too:
    >>> dameraulevenshtein('abcd', ['b', 'a', 'c', 'd', 'e'])
    2
    '''
    # codesnippet:D0DE4716-B6E6-4161-9219-2903BF8F547F
    # Conceptually, this is based on a len(seq1) + 1 * len(seq2) + 1 matrix.
    # However, only the current and two previous rows are needed at once,
    # so we only store those.
    oneago = None
    thisrow = range(1, len(seq2) + 1) + [0]
    for x in xrange(len(seq1)):
        # Python lists wrap around for negative indices, so put the
        # leftmost column at the *end* of the list. This matches with
        # the zero-indexed strings and saves extra calculation.
        twoago, oneago, thisrow = oneago, thisrow, [0] * len(seq2) + [x + 1]
        for y in xrange(len(seq2)):
            delcost = oneago[y] + 1
            addcost = thisrow[y - 1] + 1
            subcost = oneago[y - 1] + (seq1[x] != seq2[y])
            thisrow[y] = min(delcost, addcost, subcost)
            # This block deals with transpositions
            if (x > 0 and y > 0 and seq1[x] == seq2[y - 1]
                    and seq1[x - 1] == seq2[y] and seq1[x] != seq2[y]):
                thisrow[y] = min(thisrow[y], twoago[y - 2] + 1)
    return thisrow[len(seq2) - 1]


@MemoryCached
def soundex(name, length=4):
    ''' soundex module conforming to Knuth's algorithm
        implementation 2000-12-24 by Gregory Jorgensen

        source : http://code.activestate.com/recipes/52213-soundex-algorithm/
        license: public domain
    '''

    # digits holds the soundex values for the alphabet
    digits = '01230120022455012623010202'
    sndx = ''
    fc = ''

    # translate alpha chars in name to soundex digits
    for c in name.upper():
        if c.isalpha():
            if not fc:
                fc = c   # remember first letter
            d = digits[ord(c) - ord('A')]
            # duplicate consecutive soundex digits are skipped
            if not sndx or (d != sndx[-1]):
                sndx += d

    # replace first digit with first alpha character
    sndx = fc + sndx[1:]

    # remove all 0s from the soundex code
    sndx = sndx.replace('0', '')

    # return soundex code padded to length characters
    return (sndx + (length * '0'))[:length]


class VectorSpaceModel(object):
    ''' a class used for vector space representations '''

    def __init__(self, tokens, binary=False):
        if not binary:
            self.v = self._addTokens(tokens)
        else:
            self.v = self._binaryAddTokens(tokens)

    def _addTokens(self, tokens):
        ''' adds the tokens to a vsm representation '''
        return Counter(tokens)

    def _binaryAddTokens(self, tokens):
        ''' adds tokens to a binary vsm representation '''
        return Counter(set(tokens))

    def __str__(self):
        return str(self.v)

    @staticmethod
    def createVSMRepresentation(v1, v2):
        ''' creates the VSM representation for the two vectors
            to compare '''
        complete_token_list = set(v1.keys() + v2.keys())
        vsm1 = [v1[k] for k in complete_token_list]
        vsm2 = [v2[k] for k in complete_token_list]
        return vsm1, vsm2

    def __mul__(self, o):
        ''' returns the dot-product of two vectors '''
        v1, v2 = self.createVSMRepresentation(self.v, o.v)
        return vdot(v1, v2) / (norm(v1) * norm(v2))

# --------------------------------------------------------------------
# UNITTESTS
# --------------------------------------------------------------------


from unittest import TestCase


def testMetaPhone():
    ''' compares output produced by the advas metaphone
        module with the postgres fuzzymatch functions
        @warning: the used metaphone implementation _does not_ correspond
                  to the one used in postgresql.
     '''
    from warnings import warn
    warn("the used metaphone implementation _does not_ correspond to the one use in postgres")
    assert metaphone('Albert') == 'albrt'
    assert metaphone('John') == 'jhn'
    assert metaphone('Microprocessor') == 'mkrprksr'


def testNysiis():
    ''' tests the New York State Identification and Intelligence
        Algorithm (NYSIIS) code for the given term '''
    assert nysiis("Albert").lower() == 'albard'


def testCaverphone():
    ''' tests the caverphone algorithm '''
    assert caverphone("Albert") == "APT1111111"


def testSoundex():
    ''' compares the output of the soundex algorithm with
        the output provided by postgres '''
    assert soundex("Knuth") == "K530"
    assert soundex("Weichselbraun") == "W224"
    assert soundex("Smith") == "S530"
    assert soundex("Albert") == "A416"

    assert soundex("Weichselbraun", 8) == "W2241650"


def testWordSimilarity():
    ''' tests the per word similarity '''
    assert wordSimilarity("Ana Toth", "Toth Ana", lev) == 0
    assert wordSimilarity("Anna Toth", "Toth Ana", lev) == 0.5

    assert wordSimilarity("Anna Toth", "Toht Ana", lev) == 3. / 2
    assert wordSimilarity("Anna Toth", "Toht Ana", damerauLev) == 1.0


def testLevenshteinDistance():
    assert lev("anton", "ana") == 3
    assert lev("anna", "ana") == 1
    assert lev("alfred", "alfred") == 0
    assert lev("tothi", "alfredKurz") == 10
    assert lev("maria", "marion") == 2
    assert lev("safety measures", "safety measures") == 0


def testDamerauLev():
    ''' tests the Damerau-Levenshtein distance
        http://en.wikipedia.org/wiki/Damerau%E2%80%93Levenshtein_distance
    '''
    assert damerauLev("jump", "jupm") == 1
    assert damerauLev("julius", "julius") == 0
    assert damerauLev("the city of perth", "") == len("the city of perth")
    assert damerauLev("the city of perth", "teh city of perht") == 2


class TestVSM(TestCase):

    def testVSM(self):
        v1 = VectorSpaceModel(('albert', 'jasna', 'perth'))
        v2 = VectorSpaceModel(('perth', 'jasna', 'parik'))
        self.assertAlmostEqual(v1 * v1, 1.0)
        self.assertAlmostEqual(v2 * v2, 1.0)

        self.assertAlmostEqual(v1 * v2, 2 / 3.)

    def testBinaryVersusStandardVSM(self):
        v1 = VectorSpaceModel(('perth', 'perth', 'jasna'), binary=False)
        v2 = VectorSpaceModel(('perth', 'jasna', 'jasna'), binary=False)
        v3 = VectorSpaceModel(('perth', 'perth', 'jasna'), binary=True)
        v4 = VectorSpaceModel(('perth', 'jasna', 'jasna'), binary=True)

        self.assertAlmostEqual(v1 * v2, 4 / 5.)
        self.assertAlmostEqual(v3 * v4, 1.0)



#!/usr/bin/env python
"""
 @package eWRT.ws.geonames.util.georesolve
 fetches for an ContentID or GazEntry-ID where it is located
 e.g. for Vienna: Europe/Austria/Vienna
"""

# (C)opyrights 2009 by Heinz Lang <heinz.lang@wu.ac.at>
#                      Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys
from eWRT.access.db import PostgresqlDb
from eWRT.util.cache import MemoryCached
from eWRT.config import DATABASE_CONNECTION
# from warnings import warn

MIN_POPULATION = 5000


class GazetteerEntryNotFound(Exception):
    """ @class GazetteerEntryNotFound
        Base class for gazetteer lookup errors 
    """

    def __init__(self, id, query):
        self.id = id
        self.query = query
        print id, query

    def __str__(self):
        return "Gazetteer lookup for entity-id '%s' failed." % (self.id)


class GazetteerNameNotFound(Exception):
    """ @class GazetteerNameNotFound
        This exception is thrown if a lookup name has not been found in the gazetteer
    """

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return "Gazetteer lookup of name '%s' failed." % (self.name)


class Gazetteer(object):
    # sorting by population is a workaround for entries with multiple parents
    # (without the sorting loops occure)
    QUERY_HAS_PARENT = '''
        SELECT parent_id 
        FROM gazetteerentity ga 
        JOIN locatedin ON (ga.id = locatedin.child_id)
        JOIN gazetteerentity gb ON (gb.id = locatedin.parent_id)
        WHERE child_id = %d order by gb.population DESC LIMIT 1'''

    QUERY_CONTENT_ID = '''
        SELECT gazetteer_id FROM content_id_gazeteer_id WHERE content_id = %d '''

    QUERY_NAME = '''
            SELECT entity_id, ispreferredname, lang, gazetteerentry_id
            FROM gazetteerentry_ordered_names
            WHERE name LIKE '%s' '''

    DEBUG = False

    # init - establishes the db-connections
    def __init__(self):
        """ implement me """
        self.db = PostgresqlDb(**DATABASE_CONNECTION['gazetteer'])
        # PostgresqlDb.DEBUG=True
        self.db.connect()
        self.db2 = PostgresqlDb(**DATABASE_CONNECTION['geo_mapping'])
        self.db2.connect()

    @MemoryCached
    def getGeoNameFromContentID(self, content_id):
        """ returns the location of the content ID
            @param content_id
            @return list of locaions, e.g. ['Europa', 'France', 'Centre']
        """

        query = self.QUERY_CONTENT_ID % content_id
        result = self.db2.query(query)

        if result == []:
            return 'ContentID not found!'
        else:
            gaz_id = result[0]['gazetteer_id']
            return Gazetteer.getGeoNameFromGeoId(self, gaz_id)

    # returns the location of the GazetteerEntry ID
    # @param gazetteer-entry ID
    # @return list of locations, e.g. ['Europa', 'France', 'Centre']
    @MemoryCached
    def getGeoNameFromGeoId(self, gazetteer_id):
        result = self.__getLocationTree(gazetteer_id)
        result.reverse()

        if result == []:
            return 'GazetteerID not found!'
        else:
            return result

    @MemoryCached
    def getGeoNameFromString(self, name):
        """ returns the geoname for the given string
            @param string
            @return a list of tuples (population, location) 
        """
        res = set()
        query = '''SELECT entity_id, population FROM gazetteerentry JOIN hasname ON (gazetteerentry.id = hasname.entry_id) 
                  JOIN gazetteerentity ON (gazetteerentity.id=hasname.entity_id) WHERE name = '%%s' AND (population > %d or feature_code in ('ADM1', 'ADM2', 'ADM3')) ''' % (MIN_POPULATION)
        for result in self.db.query(query % name.replace("'", "''")):
            try:
                tmp = Gazetteer.getGeoNameFromGeoId(self, result['entity_id'])
                res.add((result['population'], tuple(tmp)))
            except GazetteerEntryNotFound:
                pass

        return list(res)

    # recursive function to build the full location tree
    # @param id
    # @returns list of locations
    def __getLocationTree(self, id):
        geoPath = [self.__getPreferredGeoName(id)]
        geoIdPath = [id]

        while id:
            parentLocationEntity = self.__hasParent(id)
            if parentLocationEntity:
                parentLocationName = self.__getPreferredGeoName(
                    parentLocationEntity)
                if parentLocationEntity in geoIdPath:
                    print "%s in %s" % (parentLocationName, geoPath)
                    break
                geoPath.append(parentLocationName)

            geoIdPath.append(parentLocationEntity)
            id = parentLocationEntity

        return geoPath

    # gets the preferred name for the location
    # @param id
    # @returns preferred name
    def __getPreferredGeoName(self, id):
        """ returns the preferred entry name for the given
            entity id
            @param[in] entity_id 
            @returns the geo entity's name
        """

        query = '''SELECT name FROM vw_gazetteer_tng WHERE id=%d 
                      ORDER BY 
                            (lang='en' and lang is not null) DESC,
                            (short=TRUE and short IS NOT NULL) DESC,
                            preferred DESC
                      LIMIT 1''' % (id)
        result = self.db.query(query)
        if not result:
            raise GazetteerEntryNotFound(id, query)

        return Gazetteer.DEBUG and result[0]['name'] + "(%d)" % (id) or result[0]['name']

    # checks if the given ID has a parent
    # @param ID of the child
    # @return false or ID of the parent
    def __hasParent(self, child_id):
        query = self.QUERY_HAS_PARENT % child_id
        result = self.db.query(query)

        # todo: is it necessary, that this functions can process multiple parents?
        # multiple parents (!)
        if result.__len__() > 1:
            print '### result > 1 ###'
            print '    child_id:  %s' % child_id
            print '    parent_id: %s ' % [e['parent_id'] for e in result]

        # todo: does it make sense to fetch infinite loops
        if result == []:
            return 0
        else:
            return result[0]['parent_id']

    @MemoryCached
    def __getNameGeoId(self, name):
        """ returns the possible geoids for the given name 
            @param[in] name
            @returns a list of geoids
        """
        query = "SELECT DISTINCT entity_id FROM vw_entry_id_has_name WHERE name='%s'" % (
            name.replace("'", "''"))
        return [r['entity_id'] for r in self.db.query(query)]

    def getGeoIdFromGeoUrl(self, geoUrl):
        """ returns the geoId forv the given geoUrl 
            @param[in] geoUrl String or List containing the geoUrl
            @returns the geoId
        """
        if isinstance(geoUrl, str):
            geoUrl = geoUrl.split("/")

        join = []
        where = []
        for nr, name in enumerate(geoUrl):
            join.append("JOIN locatedin L%d ON (A%d.id = L%d.parent_id) JOIN gazetteerentity A%d ON (A%d.id = L%d.child_id)" % (
                nr, nr, nr, nr + 1, nr + 1, nr))
            where.append("A%d.id IN (%s)" % (nr, ", ".join(
                map(str, self.__getNameGeoId(self, name)))))

        query = "SELECT A%d.id AS id FROM gazetteerentity A0 %s WHERE %s;" % (
            nr, " ".join(join[:-1]), " AND ".join(where))
        return [int(r['id']) for r in self.db.query(query)]

    def getGeoDict(self, geoId):
        """ returns a dictinary with all information about the given geoId """
        print geoId
        query = "SELECT * FROM gazetteerentity WHERE id = %s" % geoId
        res = self.db.query(query)
        if len(res) > 0:
            return dict(res[0])
        else:
            return {}


class TestGazeteer(object):

    def __init__(self):
        self.gazetteer = Gazetteer()

    def testUniqueStringResolver(self):
        getName = Gazetteer.getGeoNameFromString
        for name, entries in (('Lainach', 0), ('Vienna', 4), ('Madrid', 5), ('Graz', 1), ('Poland', 2), ('Geneva', 5)):
            assert len(getName(self.gazetteer, name)) == entries

    def testContentIdResolver(self):
        assert Gazetteer.getGeoNameFromContentID(self.gazetteer, 86597672) == [
            'Europe', 'France', 'Centre']

    def testGetGeoIdFromGeoUrl(self):
        """ tests the resolution from geurls to geoid's """
        for geoId in (2761367,  # Vienna
                      4277145,  # Perth
                      2063523,
                      2599670,  # Bosten
                      6470560,  # New York
                      2772400):  # Linz

            geoUrl = self.gazetteer.getGeoNameFromGeoId(self.gazetteer, geoId)
            geoIdList = self.gazetteer.getGeoIdFromGeoUrl(geoUrl)
            assert len(geoIdList) == 1
            assert geoId == geoIdList[0]


if __name__ == "__main__":
    a = Gazetteer()
    if sys.argv.__len__() > 1:
        # print a.getGeoIdFromGeoUrl( "Europe/Austria/Vienna/Vienna" )
        print Gazetteer.getGeoNameFromString(a, sys.argv[1])
    else:
        # print Gazetteer.getGeoNameFromContentID(a, 86597672)
        # print Gazetteer.getGeoNameFromContentID(a, 90160635)
        print Gazetteer.getGeoNameFromString(a, 'Vienna')
        print Gazetteer.getGeoNameFromString(a, "Lainach")
        print Gazetteer.getGeoNameFromString(a, "Spittal an der Drau")
        print Gazetteer.getGeoNameFromString(a, "Salzburg")
        print Gazetteer.getGeoNameFromString(a, "London")



#!/usr/bin/env python

''' pickelIterator '''

# (C)opyrights 2008 - 2015 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = "GPL"

import gzip
from binascii import b2a_base64, a2b_base64
try:
    from cPickle import dumps, loads
except ImportError:
    from pickle import dumps, loads


class AbstractIterator(object):
    '''
    Abstract Iterator class used to implement ReadPickleIterator
    and WritePickleIterator
    '''

    def __init__(self, fname, file_mode=None):
        self.fname = self.get_filename(fname)
        self.f = gzip.open(
            self.fname, file_mode) if file_mode else gzip.open(self.fname)

    def __iter__(self):
        return self

    def __next__(self):
        raise NotImplementedError

    def next(self):
        ''' Python 2 compatibility '''
        return self.__next__()

    def close(self):
        self.f.close()

    @classmethod
    def get_filename(cls, fname):
        return fname if fname.endswith('.gz') else fname + '.gz'


class WritePickleIterator(AbstractIterator):
    ''' writes pickeled elements (available as iterator) to a file '''

    def __init__(self, fname):
        AbstractIterator.__init__(self, fname, file_mode='w')

    def dump(self, obj):
        ''' dumps the following object to the pickle file '''
        p = b2a_base64(dumps(obj))
        self.f.write(p)


class ReadPickleIterator(AbstractIterator):
    ''' provides an iterator over pickeled elements '''

    def __init__(self, fname):
        AbstractIterator.__init__(self, fname)

    def __iter__(self):
        return self

    def __next__(self):
        ''' returns the next pickled element in the file '''
        line = self.f.readline()
        if not line:
            raise StopIteration

        return loads(a2b_base64(line))



#!/usr/bin/env python

""" __init__.py
    retrieves WikiPedia articles """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

from urllib import quote
from urllib2 import HTTPError

from eWRT.access.http import Retrieve


WIKIPEDIA_SEARCH_QUERY = 'http://%s.wikipedia.org/wiki/%s'

class WikiPedia(object):
    """ returns a wikipedia article """

    def __init__(self):
        self.r = Retrieve( WikiPedia.__name__ )
    
    def getDescriptor(self, synonym, lang='en'):
        """ returns the descriptor for the given synonym in the diven language """
        assert( len(lang)==2 )
        try:
            result = self.getWikipediaSearchResults(synonym, lang)
            return result[0]
        except (HTTPError, IndexError):
            return None


    def getWikipediaSearchResults(self, term, lang):
        """ returns a list of wikipedia search results for the given term 
            or None if nothing was found 
        """
        search_query = WIKIPEDIA_SEARCH_QUERY % (lang, quote(term) )
        f=self.r.open(search_query)
        results = WikiPedia._parse_wikipedia_search_results( f.read() )
        f.close()

        return results

    @staticmethod
    def _parse_wikipedia_search_results( text ):
        result = []
        for line in text.split("\n"):
            # only consider lines containing search results
            if not "class='searchresult'" in line: continue

            (prefix, tmp) = line.split("title=\"", 1)
            (descriptor, suffix ) = tmp.split("\"", 1)

            result.append(descriptor)

        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on 23.10.2014

.. codeauthor:: Heinz-Peter Lang <lang@weblyzard.com>

.. seealso:: 

    `Getting started with Microsoft Translator <http://blogs.msdn.com/b/translation/p/gettingstarted1.aspx>`_

    `Translator Language Codes <http://msdn.microsoft.com/en-us/library/hh456380.aspx>`_

'''
import json
import requests

from six import string_types
from datetime import datetime, timedelta
from urllib import urlencode

from eWRT.ws import AbstractWebSource


TOKEN_URL = 'https://datamarket.accesscontrol.windows.net/v2/OAuth2-13'
API_URL = 'http://api.microsofttranslator.com/v2/Ajax.svc/Translate?'
REFRESH_TOKEN_INTERVAL = 10  # in minutes


class BingTranslator(AbstractWebSource):
    NAME = 'bing_translate'
    SUPPORTED_PARAMS = ('text', 'target_language', 'source_language')

    def __init__(self, client_id, client_secret, api_url=None, token_url=None):
        ''' sets the credentials for Bing Translator 
        .. todo:: store the urls in the database as welll !!! 

        '''
        self.client_id = client_id
        self.client_secret = client_secret
        self.api_url = api_url if api_url else API_URL
        self.token_url = token_url if token_url else TOKEN_URL
        self._access_token = None
        self._next_refresh = None

    def search_documents(self, search_terms, source_language, target_language):
        ''' translates the `search_terms` from `source_language to the 
        `target_language`
        :returns: iterator with the translated text
        '''
        if isinstance(search_terms, string_types):
            search_terms = [search_terms]

        for search_term in search_terms:
            translation = self.translate(text=search_term,
                                         target_language=target_language,
                                         source_language=source_language)

            yield {'text': search_term,
                   'translations': [(translation.strip(),
                                     source_language,
                                     target_language)]}

    def translate(self, text, target_language, source_language):
        ''' tranlates the `text` in `source_language` to the `target_language`
        :param text: text to translate
        :type text: str or unicode
        :param target_language: language to translate to (iso-code)
        :param source_language: language of the given text (iso-code)
        :returns: translated text as string
        '''
        params = {'text': text,
                  'from': source_language,
                  'to': target_language
                  }
        headers = {'Authorization': 'Bearer %s' % self.access_token}
        resp = requests.get(self.api_url + urlencode(params), headers=headers)
        return resp.text.replace('"', '').replace(u'\ufeff', '')

    def get_new_access_token(self):
        params = {'grant_type': 'client_credentials',
                  'client_id': self.client_id,
                  'client_secret': self.client_secret,
                  'scope': 'http://api.microsofttranslator.com'}

        resp = requests.post(self.token_url, data=urlencode(params))
        return json.loads(resp.text)

    def get_access_token(self):
        now = datetime.now()

        too_old = False if not self._next_refresh else now > self._next_refresh

        if too_old or not self._access_token:
            self._access_token = self.get_new_access_token()
            self._next_refresh = now + \
                timedelta(minutes=REFRESH_TOKEN_INTERVAL)

        return self._access_token['access_token']

    access_token = property(get_access_token)



#!/usr/bin/env python
"""
 @package eWRT.input.conv.doc
 converts Microsoft Word documents into text
"""

# (C)opyrights 2009-2012 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from eWRT.config import CMD_CONV
from eWRT.util.execute import pipe_content


class HtmlToText(object):
    """ converts HTML into text
        requires a converter
    """

    @staticmethod
    def getText(word_document_content):
        """ @param[in] word_document_content the content of the html page to convert 
            @param[in] encoding the document encoding 
            @returns the text representation of the Web page
        """
        _, html = pipe_content(CMD_CONV['doc'], word_document_content)
        return html



# -*- coding: UTF-8 -*-
#!/usr/bin/env python

''' .. module:: eWRT.ws.rest
    .. moduleauthor:: Albert Weichselbraun <weichselbraun@weblyzard.com>
    .. moduleauthor:: Heinz-Peter Lang <lang@weblyzard.com>

    eWRT REST Client barebone with support for authentificated https requests
'''
import traceback
import logging
import random

try:
    # urllib2 is merged into urllib in python3 (SV)
    from urllib.error import HTTPError
except:
    from urllib2 import HTTPError  # python2

import urllib

try:
    from urllib.parse import urlsplit, urlunsplit  # porting to python 3.4 (SV)
except:
    from urlparse import urlsplit, urlunsplit  # python2

from six import string_types
from json import dumps, loads
from functools import partial
from socket import setdefaulttimeout

from eWRT.access.http import Retrieve


# set higher timeout values
WS_DEFAULT_TIMEOUT = 900

logger = logging.getLogger('eWRT.ws.rest')


class RESTClient(object):
    '''
    class:: RESTClient
    '''

    def __init__(self, service_url, user=None, password=None,
                 authentification_method='basic',
                 module_name='eWRT.REST', default_timeout=WS_DEFAULT_TIMEOUT):
        ''' :param service_url: the base url of the web service
            :param modul_name: the module name to add to the USER AGENT
                               description (optional)
            :param user: username
            :param password: password
            :param authentification_method: authentification method to use
                                            ('basic'*, 'digest').
        '''
        # remove superfluous slashes, if required
        self.service_url = service_url[:-1] if service_url.endswith("/") \
            else service_url
        self.user = user
        self.password = password

        if not default_timeout:
            default_timeout = WS_DEFAULT_TIMEOUT

        url_obj = Retrieve(module_name, sleep_time=0,
                           default_timeout=default_timeout)
        self.retrieve = partial(url_obj.open,
                                user=user,
                                pwd=password,
                                authentification_method=authentification_method
                                )

    def _json_request(self, url, parameters=None, return_plain=False,
                      json_encode_arguments=True,
                      content_type='application/json'):
        ''' performs the given json request
        :param url: the url to query
        :param parameters: optional paramters
        :param return_plain: whether to return the result without prior
                             deserialization using json.load (False*)
        :param json_encode_arguments: whether to json encode the parameters
                                      (True*)
        :param content_type: one of 'application/json', 'application/xml'
        '''
        if parameters:
            handle = self.retrieve(
                url,
                dumps(parameters) if json_encode_arguments else parameters,
                {'Content-Type': content_type})
        else:
            handle = self.retrieve(url)

        response = handle.read()
        if response:
            return response if return_plain else loads(response.decode('utf8'))
        else:
            # this will also return empty list, dicts ...
            return response

    @staticmethod
    def get_request_url(service_url, command, identifier=None,
                        query_parameters=None):
        '''
        Returns the request url given the command and query parameters
        :param base_url: the base url of the web service
        :param command: the command to execute at the web service
        :param identifier: an optional identifier (e.g. batch_id, ...)
        :param query_parameters: query parameters to include in the url
                                 (e.g. execute?debug=True
        :rtype: the complete request url
        '''
        # remove superfluous slashes
        if command.startswith("/"):
            command = command[1:]

        url = '%s/%s/%s' % (service_url, command, identifier) \
            if identifier else "%s/%s" % (service_url, command)

        # add query string, if necessary
        if query_parameters:
            try:
                url = url + "?" + urllib.parse.urlencode(query_parameters,
                                                         doseq=True)
            except:
                url = url + "?" + urllib.urlencode(query_parameters,
                                                   doseq=True)

        return url

    def execute(self, command, identifier=None, parameters=None,
                return_plain=False, json_encode_arguments=True,
                query_parameters=None, content_type='application/json'):
        ''' executes a json command on the given web service
        :param command: the command to execute
        :param identifier: an optional identifier (e.g. batch_id, ...)
        :param parameters: optional post paramters
        :param return_plain: return the result without prior deserialization
                             using json.load (False*)
        :param json_encode_arguments: whether to json encode the parameters
        :param query_parameters: optional query parameters
        :rtype: the query result
        '''
        url = self.get_request_url(self.service_url, command, identifier,
                                   query_parameters)

        logger.debug('requesting url %s' % url)

        return self._json_request(url, parameters, return_plain,
                                  json_encode_arguments, content_type)

class MultiRESTClient(object):
    ''' allows multiple URLs for access REST services '''
    MAX_BATCH_SIZE = 500
    URL_PATH = None

    def __init__(self, service_urls, user=None, password=None,
                 default_timeout=WS_DEFAULT_TIMEOUT, use_random_server=False):

        self._service_urls = self.fix_urls(service_urls, user, password)

        if use_random_server:
            random.shuffle(self._service_urls)

        self.clients = self._connect_clients(self._service_urls,
                                             default_timeout=default_timeout)

    def is_online(self):
        try:
            self.request('status')
            return True
        except:
            return False
        
    @classmethod
    def fix_urls(cls, urls, user=None, password=None):
        ''' fixes the urls and put them into the correct format, to maintain
        the compability to the remaining platform
        :param urls: service urls
        :type urls: string or list or tuple
        :param user: username
        :param password: password
        :returns: correctly formated urls
        :rtype: list
        '''
        correct_urls = []

        if isinstance(urls, string_types):
            urls = [urls]

        for url in urls:
            if not url.endswith('/'):
                url = '%s/' % url

            if not 'rest' in url:
                if cls.URL_PATH and not url.endswith(cls.URL_PATH):
                    if cls.URL_PATH.startswith('/'):
                        cls.URL_PATH = cls.URL_PATH[1:]
                    url = '%s%s' % (url, cls.URL_PATH)

            if user and password:
                url = Retrieve.add_user_password(url, user, password)

            correct_urls.append(url)

        return correct_urls

    @classmethod
    def _connect_clients(cls, service_urls, user=None, password=None,
                         default_timeout=WS_DEFAULT_TIMEOUT):

        clients = []

        if isinstance(service_urls, str):
            service_urls = [service_urls]

        for url in service_urls:
            service_url, user, password = Retrieve.get_user_password(url)

            clients.append(RESTClient(service_url=service_url,
                                      user=user,
                                      password=password,
                                      default_timeout=default_timeout))
        return clients

    def request(self, path, parameters=None, return_plain=False,
                execute_all_services=False, json_encode_arguments=True,
                query_parameters=None, content_type='application/json',
                pass_through_exceptions=()):
        ''' performs the given json request
        @param url: the url to query
        @param parameters: optional paramters
        @param pass_through_exceptions:
            set to True, if the client shall pass through all exceptions
        @param return_plain: whether to return the result without prior
                             deserialization using json.load (False*)
        '''
        response = None
        errors = []
        for client in self.clients:
            try:
                response = client.execute(
                    command=path,
                    parameters=parameters,
                    return_plain=return_plain,
                    json_encode_arguments=json_encode_arguments,
                    query_parameters=query_parameters,
                    content_type=content_type)

                if not execute_all_services:
                    break

            except Exception as e:  # ported to python3 (SV)
                if pass_through_exceptions:
                    raise e
                else:
                    msg = 'could not execute %s %s, error %s\n%s' % (
                        client.service_url, path, e,
                        traceback.format_exc())
                    logger.warn(msg)
                    errors.append(msg)

        if len(errors) == len(self.clients):
            print ('\n'.join(errors))
            raise Exception('Could not make request to path %s: %s' % (
                path,
                '\n'.join(errors)))

        return response

    def get_service_urls(self): 
        ''' '''
        return [client.service_url for client in self.clients]
    
    @classmethod
    def get_document_batch(cls, documents, batch_size=None):
        batch_size = batch_size if batch_size else cls.MAX_BATCH_SIZE
        for i in range(0, len(documents), batch_size):
            yield documents[i:i+batch_size]
            



#!/usr/bin/env python

# (C)opyrights 2008-2014 by Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''
Pre-defined logging profiles
'''
from __future__ import print_function

import logging

from six import string_types
from os import makedirs
from os.path import exists, dirname

DEFAULT_LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'


def get_stdout_logger(name, level=logging.DEBUG):
    '''
    Returns a logger which reports to standard output
    ::param name:
        the logger's name
    '''
    logger = logging.getLogger(name)
    logger.setLevel(level)

    formatter = logging.Formatter(DEFAULT_LOG_FORMAT)

    ch = logging.StreamHandler()
    ch.setLevel(level)
    ch.setFormatter(formatter)

    logger.addHandler(ch)
    return logger


def get_file_logger(name, filename, level=logging.DEBUG):
    '''
    Returns a logger which logs to the given filename
    ::param name:
        the logger's name
    ::param filename:
        the name of the logfile
    '''
    logger = logging.getLogger(name)
    logger.setLevel(level)

    formatter = logging.Formatter(DEFAULT_LOG_FORMAT)

    ch = logging.FileHandler(filename)
    ch.setLevel(level)
    ch.setFormatter(formatter)

    logger.addHandler(ch)
    return logger


LOG_LEVELS = {'CRITICAL': logging.CRITICAL,
              'ERROR': logging.ERROR,
              'WARNING': logging.WARNING,
              'INFO': logging.INFO,
              'DEBUG': logging.DEBUG}
LOG_FORMAT = {'standard': '%(asctime)s %(levelname)s %(pathname)s:%(lineno)d.%(funcName)s()  %(message)s',
              'compact': '%(levelname)s %(message)s'}
EWRT_LOG_LVL = 65
DEFAULT_LOG_LEVEL = 'ERROR'
DEFAULT_LOG_FILE = '/tmp/eWRT.log'


def init_logging(log_file=DEFAULT_LOG_FILE, log_level=None,
                 max_bytes=102400, max_files=5, log_format=None):
    ''' inits the logger, if default log file is used, a rotating file-handler
    will be added to the logger, otherwise it adds a standard FileHandler. In
    addition to that it also streams the streams the result to stdout.
    :param log_file: log file to use
    :param log_level: log level
    :param max_bytes: maximum bytes for the RotatingFileHandler
    :param max_files: maximum number of files for the RotatingFileHandler
    :param log_format: standard or compact
    :returns: the initialized worker.
     '''
    from logging import FileHandler
    from logging.handlers import RotatingFileHandler
    logger = logging.getLogger('')
    logging.addLevelName(65, 'EWRT_INFO')

    if not log_format or log_format not in LOG_FORMAT:
        log_format = 'standard'

    if len(logger.handlers):
        logger.handlers = []

    if log_level and isinstance(log_level, string_types):
        log_level = log_level.upper()
        if log_level in LOG_LEVELS:
            log_level = LOG_LEVELS[log_level]
        else:
            print('log_level %s not found using "ERROR"' % log_level)

    if not log_level:
        log_level = DEFAULT_LOG_LEVEL

    hdlr = logging.StreamHandler()

    logger.addHandler(hdlr)
    hdlr.setLevel(log_level)
    logger.setLevel(log_level)
    formatter = logging.Formatter(LOG_FORMAT.get(log_format))
    hdlr.setFormatter(formatter)

    # setting loglevel of tldextract to ERROR to prevent extensive messages
    tld_logger = logging.getLogger('tldextract')
    tld_logger.setLevel(logging.ERROR)

    try:
        file_hdlr = None

        if log_file:
            if log_file == DEFAULT_LOG_FILE:
                file_hdlr = RotatingFileHandler(log_file,
                                                maxBytes=max_bytes,
                                                backupCount=max_files,
                                                encoding='utf-8')
            else:
                log_dir = dirname(log_file)

                if not exists(log_dir):
                    makedirs(log_dir)

                file_hdlr = FileHandler(filename=log_file,
                                        encoding='utf-8')

        if file_hdlr:
            file_hdlr.setLevel(log_level)
            file_hdlr.setFormatter(formatter)
            logger.addHandler(file_hdlr)

    except Exception as e:
        logger.error('Couldnt create LogHandler %s: %s' % (log_file, e))

    return logger



#!/usr/bin/env python
"""
 @package eWRT.ws.geonames
"""

# (C)opyrights 2009 by Albert Weichselbraun <albert@weichselbraun.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from eWRT.util.cache import MemoryCached, DiskCached
from eWRT.access.http import Retrieve
from eWRT.ws.geonames.gazetteer import Gazetteer
from eWRT.config import GEO_ENTITY_SEPARATOR


class GeoEntity(object):
    """ a geographic entity """

    def __init__(self, entityDict):
        assert isinstance(entityDict, dict)
        self.entityDict = entityDict
        self.id = entityDict['id']

    @staticmethod
    @MemoryCached
    def factory(name=None, id=None, geoUrl=None):
        """ creates geoentity objects based on the given information
            @param[in] name   of the Entity
            @param[in] id     the GeoNames id 
            @param[in] geoUrl the entity url
        """
        g = Gazetteer()
        return [GeoEntity(d) for d in g.getGeoEntityDict(name, id, geoUrl)]

    def __getitem__(self, key):
        return self.entityDict.get(key)

    def __sub__(self, geoEntity):
        """ returns the distance between the two locations 
            @param[in] the entity to compare
            @returns the distance in m
        """
        pass

    def __str__(self):
        return "GeoEntity <%s (id=%s)>" % (self.entityDict['geoUrl'], self.entityDict['id'])

    def contains(self, geoEntity):
        """ Returns true if the Object contains the given GeoEntity.
            e.g. eu>at>Carinthia contains 
                 eu>at>Carinthia, eu>at>Carinthia>Spittal/Drau and any
                 other more detailed specification.

            @param[in] geoEntity
            @returns true or false """
        # required to handle eu>Serbia versus eu>Serbia and Montenegro
        if (self['geoUrl'] + GEO_ENTITY_SEPARATOR) in (geoEntity['geoUrl'] + GEO_ENTITY_SEPARATOR):
            return True
        else:
            return False

    def highestCommonLevel(self, geoEntity):
        """ Returns the highest common level between the two GeoEntities
            e.g. eu>at>Carinthia, eu>at>Carinthia>Spittal/Drau => 3
            @param[in] geoEntity
            @returns the level
        """
        selfLevels = self['geoUrl'].split(GEO_ENTITY_SEPARATOR)
        cmpLevels = geoEntity['geoUrl'].split(GEO_ENTITY_SEPARATOR)
        for level, (l1, l2) in enumerate(zip(selfLevels, cmpLevels)):
            if l1 != l2:
                break
            level += 1

        return level

    def getState(self):
        """ Returns the state in which the given geoEntity
            is located """

        assert self['level'] >= 3
        print self['level'], self['idUrl'], self['idUrl'][2]
        return GeoEntity.factory(id=self['idUrl'][2])[0]

    def getCountry(self):
        """ Returns the country in which the given geoEntity
            is located """

        assert self['level'] >= 2
        return GeoEntity.factory(id=self['idUrl'][1])[0]

    def getContinent(self):
        """ Returns the continent in which the given
            geoEntity is located """
        assert self['level'] >= 1
        return GeoEntity.factory(id=self['idUrl'][0])[0]

    def __eq__(self, o):
        """ add's support for comparisons using == """
        return self['id'] == o['id'] or self['geoUrl'] == o['geoUrl']

    def __neq__(self, o):
        """ adds support for the != operator """
        return self['id'] != o['id']

    def __cmp__(self, o):
        return self['id'].__cmp__(o['id'])

    def __hash__(self):
        return self['id'].__hash__()


class GeoNames(object):
    """ retrieves information for GeoNames ids
        please use the helper classes in utils.georesolve
        to translate geoURLs into geoIDs 
    """

    NEIGHBOURS_SERVICE_URL = "http://ws.geonames.org/neighboursJSON?geonameId=%d"

    # required to handle cases where the factory does not return any GeoEntity
    getGeoEntity = staticmethod(
        lambda factory: factory != [] and factory[0] or factory)

    @staticmethod
    @DiskCached("./.geonames-neighbours")
    def getNeighbors(geo_entity):
        """ returns all neighbours for the given geo id
            (currently only implemented on a country level)
            @param[in] geo_entity
            @returns a list containing the neighbours of the given country """

        url = GeoNames.NEIGHBOURS_SERVICE_URL % geo_entity.id
        jsonData = eval(Retrieve('eWRT.ws.geonames').open(url, retry=5).read())
        if 'geonames' in jsonData:
            return filter(None, [GeoNames.getGeoEntity(GeoEntity.factory(id=e['geonameId'])) for e in jsonData['geonames']])
        else:
            return []


class TestGeoNames(object):

    def __init__(self):
        def g(x): return GeoEntity.factory(id=x)[0]
        self.EXAMPLE_ENTITIES = {'.ch': g(2658434),
                                 '.at': g(2782113),
                                 '.carinthia': g(2774686),
                                 '.eu': g(6255148),
                                 'villach': g(2762372),
                                 'hermagor': g(2776497),
                                 'serbia': g(6290252),
                                 'montenegro': g(863038),
                                 }

    def testGetNeighbors(self):
        geoEntity = self.EXAMPLE_ENTITIES['.ch']  # .ch
        assert set([g.id for g in GeoNames.getNeighbors(geoEntity)]) == set(
            [2782113, 3017382, 2921044, 3175395, 3042058])

    def testAreaInfo(self):
        """ verifies whether the information on a entities country is retrieved 
            correctly (currently only supported for countries """
        assert self.EXAMPLE_ENTITIES['.at'].entityDict['area'] > 0
        assert self.EXAMPLE_ENTITIES['.ch'].entityDict['area'] > 0

        assert self.EXAMPLE_ENTITIES['villach'].entityDict['area'] == None

    def testContains(self):
        geoEntity = self.EXAMPLE_ENTITIES['.at']  # .at
        assert geoEntity.contains(self.EXAMPLE_ENTITIES['.at']) == True
        assert geoEntity.contains(self.EXAMPLE_ENTITIES['.carinthia']) == True
        assert geoEntity.contains(self.EXAMPLE_ENTITIES['.eu']) == False
        assert geoEntity.contains(self.EXAMPLE_ENTITIES['.ch']) == False
        # eu>Serbia <-> eu>Serbia and Montenegro
        assert self.EXAMPLE_ENTITIES['montenegro'].contains(
            self.EXAMPLE_ENTITIES['serbia']) == False
        assert self.EXAMPLE_ENTITIES['serbia'].contains(
            self.EXAMPLE_ENTITIES['montenegro']) == False

    def testIsContained(self):
        geoEntity = self.EXAMPLE_ENTITIES['.at']  # .at
        assert self.EXAMPLE_ENTITIES['.at'].contains(geoEntity) == True
        assert self.EXAMPLE_ENTITIES['.carinthia'].contains(geoEntity) == False
        assert self.EXAMPLE_ENTITIES['.eu'].contains(geoEntity) == True
        assert self.EXAMPLE_ENTITIES['.ch'].contains(geoEntity) == False

    def testHighestCommonLevels(self):
        g = self.EXAMPLE_ENTITIES['.carinthia']
        assert g.highestCommonLevel(self.EXAMPLE_ENTITIES['.at']) == 2
        assert g.highestCommonLevel(self.EXAMPLE_ENTITIES['.eu']) == 1
        print g.highestCommonLevel(self.EXAMPLE_ENTITIES['.ch'])
        assert g.highestCommonLevel(self.EXAMPLE_ENTITIES['.ch']) == 1
        assert g.highestCommonLevel(self.EXAMPLE_ENTITIES['.carinthia']) == 3

    def testGetState(self):
        assert self.EXAMPLE_ENTITIES['villach'].getState(
        ) == self.EXAMPLE_ENTITIES['.carinthia']
        assert self.EXAMPLE_ENTITIES['hermagor'].getState(
        ) == self.EXAMPLE_ENTITIES['.carinthia']
        assert self.EXAMPLE_ENTITIES['.carinthia'].getState(
        ) == self.EXAMPLE_ENTITIES['.carinthia']

    def testGetCountry(self):
        assert self.EXAMPLE_ENTITIES['villach'].getCountry(
        ) == self.EXAMPLE_ENTITIES['.at']
        assert self.EXAMPLE_ENTITIES['hermagor'].getCountry(
        ) == self.EXAMPLE_ENTITIES['.at']
        assert self.EXAMPLE_ENTITIES['.carinthia'].getCountry(
        ) == self.EXAMPLE_ENTITIES['.at']
        assert self.EXAMPLE_ENTITIES['.at'].getCountry(
        ) == self.EXAMPLE_ENTITIES['.at']
        assert not self.EXAMPLE_ENTITIES['.at'].getCountry(
        ) == self.EXAMPLE_ENTITIES['.ch']

    def testSetProperties(self):
        """ verifies that functions such as __hash__, __cmp__ required for handling
            sets work """
        a = set(self.EXAMPLE_ENTITIES.values())
        b = set((self.EXAMPLE_ENTITIES['villach'],
                 self.EXAMPLE_ENTITIES['hermagor']))
        assert a.intersection(b) == b
        assert len(a.difference(b)) < len(a)


# if __name__ == '__main__':
#     g = GeoEntity.factory(geoUrl = 'Europe>Austria>Vienna')
#     print g, g[0]
#     print g[0].entityDict



#!/usr/bin/env python

from eWRT.input.csv import get_csv_data
from eWRT.util.module_path import get_resource

TEST_FILE = get_resource(__file__, ('test.csv', ))

def test_csv_data():
    CORRECT = ( [1,2],[2,4],[4,6] )
    with open(TEST_FILE) as f:
        for correct, computed in zip(CORRECT, get_csv_data(f,
                                                        ('int(row["a"])', 'int(row["a"])+int(row["b"])'),
                                                        'row["show"]=="True"')):
            print correct, computed
            assert correct == computed



from __future__ import print_function


def info(object, spacing=10, collapse=1):
    """Print methods and doc strings.
    Takes module, class, list, dictionary, or string."""
    methodList = [method for method in dir(
        object) if callable(getattr(object, method))]
    processFunc = collapse and (lambda s: " ".join(s.split())) or (lambda s: s)
    print("\n".join(["%s %s" %
                     (method.ljust(spacing),
                      processFunc(str(getattr(object, method).__doc__)))
                     for method in methodList]))


if __name__ == "__main__":
    print(info.__doc__)
